@@ -58,10 +58,8 @@ struct _GstGnomeVFSSrc
   /* uri, file, ... */
   GnomeVFSURI *uri;
   gchar *uri_name;
-  GnomeVFSContext *context;
   GnomeVFSHandle *handle;
   gboolean own_handle;
-  gboolean interrupted;
   GnomeVFSFileOffset curoffset; /* current offset in file */
   gboolean seekable;
 
@@ -69,6 +67,8 @@ struct _GstGnomeVFSSrc
   gboolean iradio_mode;
   gboolean http_callbacks_pushed;
 
+  GstCaps *icy_caps;
+
   gchar *iradio_name;
   gchar *iradio_genre;
   gchar *iradio_url;
@@ -86,4 +86,3 @@ G_END_DECLS
 
 #endif /* __GST_GNOME_VFS_SRC_H__ */
 
-
