@@ -35,10 +35,10 @@ G_BEGIN_DECLS
 #define GST_THEORA_DEC(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_THEORA_DEC,GstTheoraDec))
 #define GST_THEORA_DEC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_THEORA_DEC,GstTheoraDec))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_THEORA_DEC,GstTheoraDecClass))
 #define GST_IS_THEORA_DEC(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_THEORA_DEC))
-#define GST_IS_THEORA_DEC_CLASS(obj) \
+#define GST_IS_THEORA_DEC_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_THEORA_DEC))
 
 typedef struct _GstTheoraDec GstTheoraDec;
@@ -63,6 +63,7 @@ struct _GstTheoraDec
   theora_comment comment;
 
   gboolean have_header;
+  gboolean sent_newsegment;
   guint64 granulepos;
   guint64 granule_shift;
 
@@ -74,7 +75,15 @@ struct _GstTheoraDec
 
   gboolean crop;
 
+  /* list of buffers that need timestamps */
   GList *queued;
+  /* list of raw output buffers */
+  GList *output;
+  /* gather/decode queues for reverse playback */
+  GList *gather;
+  GList *decode;
+
+  GstTagList *tags;
 
   /* segment info */ /* with STREAM_LOCK */
   GstSegment segment;
