@@ -172,6 +172,7 @@ static void
 gst_theora_dec_reset (GstTheoraDec * dec)
 {
   dec->need_keyframe = TRUE;
+  dec->sent_newsegment = FALSE;
   dec->last_timestamp = -1;
   dec->granulepos = -1;
   dec->discont = TRUE;
@@ -192,6 +193,11 @@ gst_theora_dec_reset (GstTheoraDec * dec)
   g_list_foreach (dec->decode, (GFunc) gst_mini_object_unref, NULL);
   g_list_free (dec->decode);
   dec->decode = NULL;
+
+  if (dec->tags) {
+    gst_tag_list_free (dec->tags);
+    dec->tags = NULL;
+  }
 }
 
 static int
@@ -230,7 +236,7 @@ _theora_granule_frame (GstTheoraDec * dec, gint64 granulepos)
 static GstClockTime
 _theora_granule_time (GstTheoraDec * dec, gint64 granulepos)
 {
-  gint framecount;
+  gint64 framecount;
 
   /* invalid granule results in invalid time */
   if (granulepos == -1)
@@ -681,7 +687,7 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
       gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
           &start, &stop, &time);
 
-      /* we need TIME and a positive rate */
+      /* we need TIME format */
       if (format != GST_FORMAT_TIME)
         goto newseg_wrong_format;
 
@@ -689,8 +695,14 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
       gst_segment_set_newsegment_full (&dec->segment, update,
           rate, arate, format, start, stop, time);
 
-      /* and forward */
-      ret = gst_pad_push_event (dec->srcpad, event);
+      /* We don't forward this unless/until the decoder is initialised */
+      if (dec->have_header) {
+        ret = gst_pad_push_event (dec->srcpad, event);
+        dec->sent_newsegment = TRUE;
+      } else {
+        gst_event_unref (event);
+        ret = TRUE;
+      }
       break;
     }
     default:
@@ -748,7 +760,7 @@ theora_handle_comment_packet (GstTheoraDec * dec, ogg_packet * packet)
         GST_TAG_NOMINAL_BITRATE, dec->info.target_bitrate, NULL);
   }
 
-  gst_element_found_tags_for_pad (GST_ELEMENT_CAST (dec), dec->srcpad, list);
+  dec->tags = list;
 
   return GST_FLOW_OK;
 }
@@ -758,6 +770,9 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
 {
   GstCaps *caps;
   gint par_num, par_den;
+  GstFlowReturn ret = GST_FLOW_OK;
+  gboolean eret;
+  GstEvent *event;
 
   GST_DEBUG_OBJECT (dec, "fps %d/%d, PAR %d/%d",
       dec->info.fps_numerator, dec->info.fps_denominator,
@@ -782,6 +797,12 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
   GST_DEBUG_OBJECT (dec, "frame dimension %dx%d, offset %d:%d",
       dec->info.frame_width, dec->info.frame_height,
       dec->info.offset_x, dec->info.offset_y);
+  if (dec->info.pixelformat != OC_PF_420) {
+    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
+        (NULL), ("pixel formats other than 4:2:0 not yet supported"));
+
+    return GST_FLOW_ERROR;
+  }
 
   if (dec->crop) {
     /* add black borders to make width/height/offsets even. we need this because
@@ -818,8 +839,26 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
   gst_caps_unref (caps);
 
   dec->have_header = TRUE;
+  if (!dec->sent_newsegment) {
+    GST_DEBUG_OBJECT (dec, "Sending newsegment event");
 
-  return GST_FLOW_OK;
+    event = gst_event_new_new_segment_full (FALSE,
+        dec->segment.rate, dec->segment.applied_rate,
+        dec->segment.format, dec->segment.start, dec->segment.stop,
+        dec->segment.time);
+    eret = gst_pad_push_event (dec->srcpad, event);
+    if (!eret)
+      ret = GST_FLOW_ERROR;
+    dec->sent_newsegment = TRUE;
+  }
+
+  if (dec->tags) {
+    gst_element_found_tags_for_pad (GST_ELEMENT_CAST (dec), dec->srcpad,
+        dec->tags);
+    dec->tags = NULL;
+  }
+
+  return ret;
 }
 
 static GstFlowReturn
@@ -930,6 +969,10 @@ theora_dec_push_forward (GstTheoraDec * dec, GstBuffer * buf)
 
         GST_DEBUG_OBJECT (dec, "patch buffer %lld %lld", size, time);
         GST_BUFFER_TIMESTAMP (buffer) = time;
+        /* Next timestamp - this one is duration */
+        GST_BUFFER_DURATION (buffer) =
+            (outtime - gst_util_uint64_scale_int ((size - 1) * GST_SECOND,
+                dec->info.fps_denominator, dec->info.fps_numerator)) - time;
 
         if (dec->discont) {
           GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DISCONT);
@@ -986,8 +1029,8 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
     goto not_initialized;
 
   /* the second most significant bit of the first data byte is cleared 
-   * for keyframes */
-  keyframe = (packet->packet[0] & 0x40) == 0;
+   * for keyframes. We can only check it if it's not a zero-length packet. */
+  keyframe = packet->bytes && ((packet->packet[0] & 0x40) == 0);
   if (G_UNLIKELY (keyframe)) {
     GST_DEBUG_OBJECT (dec, "we have a keyframe");
     dec->need_keyframe = FALSE;
@@ -1097,9 +1140,16 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
   if (dec->frame_nr != -1)
     dec->frame_nr++;
   GST_BUFFER_OFFSET_END (out) = dec->frame_nr;
-  GST_BUFFER_DURATION (out) =
-      gst_util_uint64_scale_int (GST_SECOND, dec->info.fps_denominator,
-      dec->info.fps_numerator);
+  if (dec->granulepos != -1) {
+    gint64 cf = _theora_granule_frame (dec, dec->granulepos) + 1;
+
+    GST_BUFFER_DURATION (out) = gst_util_uint64_scale_int (cf * GST_SECOND,
+        dec->info.fps_denominator, dec->info.fps_numerator) - outtime;
+  } else {
+    GST_BUFFER_DURATION (out) =
+        gst_util_uint64_scale_int (GST_SECOND, dec->info.fps_denominator,
+        dec->info.fps_numerator);
+  }
   GST_BUFFER_TIMESTAMP (out) = outtime;
 
   if (dec->segment.rate >= 0.0)
@@ -1184,8 +1234,9 @@ theora_dec_decode_buffer (GstTheoraDec * dec, GstBuffer * buf)
     dec->last_timestamp = -1;
   }
 
-  GST_DEBUG_OBJECT (dec, "header=%02x packetno=%lld, outtime=%" GST_TIME_FORMAT,
-      packet.bytes ? packet.packet[0] : -1, packet.packetno,
+  GST_DEBUG_OBJECT (dec, "header=%02x packetno=%lld, granule pos=%"
+      G_GINT64_FORMAT ", outtime=%" GST_TIME_FORMAT,
+      packet.bytes ? packet.packet[0] : -1, packet.packetno, packet.granulepos,
       GST_TIME_ARGS (dec->last_timestamp));
 
   /* switch depending on packet type. A zero byte packet is always a data
@@ -1414,6 +1465,7 @@ theora_dec_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       theora_info_init (&dec->info);
       theora_comment_init (&dec->comment);
+      GST_DEBUG_OBJECT (dec, "Setting have_header to FALSE in READY->PAUSED");
       dec->have_header = FALSE;
       gst_theora_dec_reset (dec);
       break;
