@@ -396,6 +396,7 @@ theora_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
       enc->info.keyframe_frequency_force, enc->granule_shift);
 
   theora_enc_reset (enc);
+  enc->initialised = TRUE;
 
   gst_object_unref (enc);
 
@@ -420,14 +421,17 @@ theora_buffer_from_packet (GstTheoraEnc * enc, ogg_packet * packet,
     GstClockTime timestamp, GstClockTime duration, GstBuffer ** buffer)
 {
   GstBuffer *buf;
-  GstFlowReturn ret;
+  GstFlowReturn ret = GST_FLOW_OK;
 
-  ret = gst_pad_alloc_buffer_and_set_caps (enc->srcpad,
-      GST_BUFFER_OFFSET_NONE, packet->bytes, GST_PAD_CAPS (enc->srcpad), &buf);
-  if (ret != GST_FLOW_OK)
-    goto no_buffer;
+  buf = gst_buffer_new_and_alloc (packet->bytes);
+  if (!buf) {
+    GST_WARNING_OBJECT (enc, "Could not allocate buffer");
+    ret = GST_FLOW_ERROR;
+    goto done;
+  }
 
   memcpy (GST_BUFFER_DATA (buf), packet->packet, packet->bytes);
+  gst_buffer_set_caps (buf, GST_PAD_CAPS (enc->srcpad));
   /* see ext/ogg/README; OFFSET_END takes "our" granulepos, OFFSET its
    * time representation */
   GST_BUFFER_OFFSET_END (buf) =
@@ -453,14 +457,9 @@ theora_buffer_from_packet (GstTheoraEnc * enc, ogg_packet * packet,
   }
   enc->packetno++;
 
+done:
   *buffer = buf;
   return ret;
-
-no_buffer:
-  {
-    *buffer = NULL;
-    return ret;
-  }
 }
 
 /* push out the buffer and do internal bookkeeping */
@@ -552,19 +551,45 @@ theora_enc_sink_event (GstPad * pad, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
-      /* push last packet with eos flag */
-      while (theora_encode_packetout (&enc->state, 1, &op)) {
-        /* See comment in the chain function */
-        GstClockTime next_time = theora_granule_time (&enc->state,
-            granulepos_add (op.granulepos, 1, enc->granule_shift)) * GST_SECOND;
+      if (enc->initialised) {
+        /* push last packet with eos flag */
+        while (theora_encode_packetout (&enc->state, 1, &op)) {
+          /* See comment in the chain function */
+          GstClockTime next_time = theora_granule_time (&enc->state,
+              granulepos_add (op.granulepos, 1, enc->granule_shift)) *
+              GST_SECOND;
 
-        theora_push_packet (enc, &op, enc->next_ts, next_time - enc->next_ts);
-        enc->next_ts = next_time;
+          theora_push_packet (enc, &op, enc->next_ts, next_time - enc->next_ts);
+          enc->next_ts = next_time;
+        }
       }
       res = gst_pad_push_event (enc->srcpad, event);
       break;
+    case GST_EVENT_CUSTOM_DOWNSTREAM:
+    {
+      const GstStructure *s;
+
+      s = gst_event_get_structure (event);
+
+      if (gst_structure_has_name (s, "GstForceKeyUnit")) {
+        GstClockTime next_ts;
+
+        /* make sure timestamps increment after reseting the decoder */
+        next_ts = enc->next_ts + enc->timestamp_offset;
+
+        theora_enc_reset (enc);
+        enc->granulepos_offset =
+            gst_util_uint64_scale (next_ts, enc->fps_n,
+            GST_SECOND * enc->fps_d);
+        enc->timestamp_offset = next_ts;
+        enc->next_ts = 0;
+      }
+      res = gst_pad_push_event (enc->srcpad, event);
+      break;
+    }
     default:
       res = gst_pad_push_event (enc->srcpad, event);
+      break;
   }
   return res;
 }
@@ -610,14 +635,15 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
 
   in_time = GST_BUFFER_TIMESTAMP (buffer);
 
-  /* no packets written yet, setup headers */
   if (enc->packetno == 0) {
+    /* no packets written yet, setup headers */
     GstCaps *caps;
     GstBuffer *buf1, *buf2, *buf3;
 
     enc->granulepos_offset = 0;
     enc->timestamp_offset = 0;
 
+    GST_DEBUG_OBJECT (enc, "output headers");
     /* Theora streams begin with three headers; the initial header (with
        most of the codec setup parameters) which is mandated by the Ogg
        bitstream spec.  The second header holds any comment fields.  The
@@ -694,9 +720,8 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
     }
 
     enc->granulepos_offset =
-        gst_util_uint64_scale (GST_BUFFER_TIMESTAMP (buffer), enc->fps_n,
-        GST_SECOND * enc->fps_d);
-    enc->timestamp_offset = GST_BUFFER_TIMESTAMP (buffer);
+        gst_util_uint64_scale (in_time, enc->fps_n, GST_SECOND * enc->fps_d);
+    enc->timestamp_offset = in_time;
     enc->next_ts = 0;
   }
 
@@ -717,6 +742,7 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
     y_size = enc->info_width * enc->info_height;
 
     if (enc->width == enc->info_width && enc->height == enc->info_height) {
+      GST_LOG_OBJECT (enc, "no cropping/conversion needed");
       /* easy case, no cropping/conversion needed */
       pixels = GST_BUFFER_DATA (buffer);
 
@@ -735,6 +761,7 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
       gint cwidth, cheight;
       gint offset_x, right_x, right_border;
 
+      GST_LOG_OBJECT (enc, "cropping/conversion needed for strides");
       /* source width/height */
       width = enc->width;
       height = enc->height;
@@ -750,11 +777,13 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
       dst_y_stride = enc->info_width;
       dst_uv_stride = enc->info_width / 2;
 
-      ret = gst_pad_alloc_buffer_and_set_caps (enc->srcpad,
-          GST_BUFFER_OFFSET_NONE, y_size * 3 / 2, GST_PAD_CAPS (enc->srcpad),
-          &newbuf);
-      if (ret != GST_FLOW_OK)
+      newbuf = gst_buffer_new_and_alloc (y_size * 3 / 2);
+      if (!newbuf) {
+        ret = GST_FLOW_ERROR;
         goto no_buffer;
+      }
+      GST_BUFFER_OFFSET (newbuf) = GST_BUFFER_OFFSET_NONE;
+      gst_buffer_set_caps (newbuf, GST_PAD_CAPS (enc->srcpad));
 
       dest_y = yuv.y = GST_BUFFER_DATA (newbuf);
       dest_u = yuv.u = yuv.y + y_size;
@@ -848,9 +877,8 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
     if (theora_enc_is_discontinuous (enc, buffer)) {
       theora_enc_reset (enc);
       enc->granulepos_offset =
-          gst_util_uint64_scale (GST_BUFFER_TIMESTAMP (buffer), enc->fps_n,
-          GST_SECOND * enc->fps_d);
-      enc->timestamp_offset = GST_BUFFER_TIMESTAMP (buffer);
+          gst_util_uint64_scale (in_time, enc->fps_n, GST_SECOND * enc->fps_d);
+      enc->timestamp_offset = in_time;
       enc->next_ts = 0;
       enc->next_discont = TRUE;
     }
@@ -891,7 +919,6 @@ header_push:
   }
 no_buffer:
   {
-    gst_buffer_unref (buffer);
     return ret;
   }
 data_push:
@@ -943,6 +970,7 @@ theora_enc_change_state (GstElement * element, GstStateChange transition)
       theora_info_clear (&enc->info);
 
       theora_enc_clear (enc);
+      enc->initialised = FALSE;
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
