@@ -32,10 +32,16 @@
  * Last reviewed on 2006-09-27 (0.10.12)
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <string.h>
 
 #include "gstbaseaudiosrc.h"
 
+#include "gst/gst-i18n-plugin.h"
+
 GST_DEBUG_CATEGORY_STATIC (gst_base_audio_src_debug);
 #define GST_CAT_DEFAULT gst_base_audio_src_debug
 
@@ -56,8 +62,18 @@ enum
   PROP_LATENCY_TIME,
 };
 
-#define _do_init(bla) \
-    GST_DEBUG_CATEGORY_INIT (gst_base_audio_src_debug, "baseaudiosrc", 0, "baseaudiosrc element");
+static void
+_do_init (GType type)
+{
+  GST_DEBUG_CATEGORY_INIT (gst_base_audio_src_debug, "baseaudiosrc", 0,
+      "baseaudiosrc element");
+
+#ifdef ENABLE_NLS
+  GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
+      LOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif /* ENABLE_NLS */
+}
 
 GST_BOILERPLATE_FULL (GstBaseAudioSrc, gst_base_audio_src, GstPushSrc,
     GST_TYPE_PUSH_SRC, _do_init);
@@ -522,6 +538,7 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
   gint bps;
   GstRingBuffer *ringbuffer;
   guint read;
+  GstClockTime timestamp;
 
   ringbuffer = src->ringbuffer;
 
@@ -550,6 +567,8 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
     sample = gst_base_audio_src_get_offset (src);
   }
 
+  GST_DEBUG_OBJECT (src, "reading from sample %" G_GUINT64_FORMAT, sample);
+
   /* get the number of samples to read */
   samples = length / bps;
 
@@ -579,7 +598,8 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
     GST_WARNING_OBJECT (src,
         "create DISCONT of %" G_GUINT64_FORMAT " samples at sample %"
         G_GUINT64_FORMAT, sample - src->next_sample, sample);
-    GST_ELEMENT_WARNING (src, CORE, CLOCK, (NULL),
+    GST_ELEMENT_WARNING (src, CORE, CLOCK,
+        (_("Can't record audio fast enough")),
         ("dropped %" G_GUINT64_FORMAT " samples", sample - src->next_sample));
     GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
   }
@@ -588,8 +608,10 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
    * where we are slaved to another clock. We currently refuse to accept
    * any other clock than the one we provide, so this code is fine for
    * now. */
-  GST_BUFFER_TIMESTAMP (buf) = gst_util_uint64_scale_int (sample,
-      GST_SECOND, ringbuffer->spec.rate);
+  timestamp =
+      gst_util_uint64_scale_int (sample, GST_SECOND, ringbuffer->spec.rate);
+
+  GST_BUFFER_TIMESTAMP (buf) = timestamp;
   src->next_sample = sample + samples;
   GST_BUFFER_DURATION (buf) = gst_util_uint64_scale_int (src->next_sample,
       GST_SECOND, ringbuffer->spec.rate) - GST_BUFFER_TIMESTAMP (buf);
