@@ -29,6 +29,7 @@
 #include <gst/audio/multichannel.h>
 
 #include <string.h>
+#include <math.h>
 
 GST_DEBUG_CATEGORY_EXTERN (riff_debug);
 #define GST_CAT_DEFAULT riff_debug
@@ -521,11 +522,20 @@ gst_riff_create_video_caps (guint32 codec_fcc,
       break;
 
     case GST_MAKE_FOURCC ('T', 'S', 'C', 'C'):
-    case GST_MAKE_FOURCC ('t', 's', 'c', 'c'):
-      caps = gst_caps_new_simple ("video/x-camtasia", NULL);
+    case GST_MAKE_FOURCC ('t', 's', 'c', 'c'):{
+      if (strf) {
+        gint depth = (strf->bit_cnt != 0) ? (gint) strf->bit_cnt : 24;
+
+        caps = gst_caps_new_simple ("video/x-camtasia", "depth", G_TYPE_INT,
+            depth, NULL);
+      } else {
+        /* template caps */
+        caps = gst_caps_new_simple ("video/x-camtasia", NULL);
+      }
       if (codec_name)
         *codec_name = g_strdup ("TechSmith Camtasia");
       break;
+    }
 
     case GST_MAKE_FOURCC ('V', 'C', 'R', '1'):
       caps = gst_caps_new_simple ("video/x-ati-vcr",
@@ -631,12 +641,16 @@ gst_riff_create_video_caps (guint32 codec_fcc,
           GST_BUFFER_SIZE (palette));
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
-      gint n;
-      guint32 *data = (guint32 *) GST_BUFFER_DATA (copy);
+      {
+        guint8 *data = GST_BUFFER_DATA (copy);
+        gint n;
 
-      /* own endianness */
-      for (n = 0; n < num_colors; n++)
-        data[n] = GUINT32_FROM_LE (data[n]);
+        /* own endianness */
+        for (n = 0; n < num_colors; n++) {
+          GST_WRITE_UINT32_BE (data, GST_READ_UINT32_LE (data));
+          data += sizeof (guint32);
+        }
+      }
 #endif
       gst_caps_set_simple (caps, "palette_data", GST_TYPE_BUFFER, copy, NULL);
       gst_buffer_unref (copy);
@@ -739,16 +753,30 @@ gst_riff_create_audio_caps (guint16 codec_id,
 
   switch (codec_id) {
     case GST_RIFF_WAVE_FORMAT_PCM:     /* PCM */
+      channels_max = 8;
+
       if (strf != NULL) {
         gint ba = strf->blockalign;
         gint ch = strf->channels;
-        gint ws = strf->size;
+        gint wd = ba * 8 / ch;
+        gint ws;
+
+        if (strf->size > 32) {
+          GST_WARNING ("invalid depth (%d) of pcm audio, overwriting.",
+              strf->size);
+          strf->size = 8 * (guint) ceil (wd / 8.0);
+        }
+        /* in riff, the depth is stored in the size field but it just means that
+         * the _least_ significant bits are cleared. We can therefore just play
+         * the sample as if it had a depth == width */
+        /* For reference, the actual depth is in strf->size */
+        ws = wd;
 
         caps = gst_caps_new_simple ("audio/x-raw-int",
             "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
             "channels", G_TYPE_INT, ch,
-            "width", G_TYPE_INT, (int) (ba * 8 / ch),
-            "depth", G_TYPE_INT, ws, "signed", G_TYPE_BOOLEAN, ws != 8, NULL);
+            "width", G_TYPE_INT, wd,
+            "depth", G_TYPE_INT, ws, "signed", G_TYPE_BOOLEAN, wd != 8, NULL);
 
         /* Add default MS channel layout if we have more than 2 channels,
          * but the layout isn't specified like with WAVEEXT below. Not sure
@@ -765,6 +793,9 @@ gst_riff_create_audio_caps (guint16 codec_id,
             case 6:
               channel_mask = 0x3f;
               break;
+            case 8:
+              channel_mask = 0xff;
+              break;
             default:
               GST_WARNING ("don't know default layout for %d channels", ch);
               channel_mask = 0;
@@ -783,8 +814,7 @@ gst_riff_create_audio_caps (guint16 codec_id,
         caps = gst_caps_from_string ("audio/x-raw-int, "
             "endianness = (int) LITTLE_ENDIAN, "
             "signed = (boolean) { true, false }, "
-            "width = (int) { 8, 16, 24, 32 }, "
-            "depth = (int) { 8, 16, 24, 32 }");
+            "width = (int) { 8, 16, 24, 32 }, " "depth = (int) [ 1, 32 ]");
       }
       if (codec_name && strf)
         *codec_name = g_strdup_printf ("Uncompressed %d-bit PCM audio",
@@ -799,6 +829,59 @@ gst_riff_create_audio_caps (guint16 codec_id,
       block_align = TRUE;
       break;
 
+    case GST_RIFF_WAVE_FORMAT_IEEE_FLOAT:
+      channels_max = 8;
+
+      if (strf != NULL) {
+        gint ba = strf->blockalign;
+        gint ch = strf->channels;
+        gint wd = ba * 8 / ch;
+
+        caps = gst_caps_new_simple ("audio/x-raw-float",
+            "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+            "channels", G_TYPE_INT, ch, "width", G_TYPE_INT, wd, NULL);
+
+        /* Add default MS channel layout if we have more than 2 channels,
+         * but the layout isn't specified like with WAVEEXT below. Not sure
+         * if this is right, but at least it makes sound output work at all
+         * in those cases. Somebody with a a 5.1 setup should double-check
+         * with chan-id.wav */
+        if (ch > 2) {
+          guint32 channel_mask;
+
+          switch (ch) {
+            case 4:
+              channel_mask = 0x33;
+              break;
+            case 6:
+              channel_mask = 0x3f;
+              break;
+            case 8:
+              channel_mask = 0xff;
+              break;
+            default:
+              GST_WARNING ("don't know default layout for %d channels", ch);
+              channel_mask = 0;
+              break;
+          }
+
+          if (channel_mask) {
+            GST_DEBUG ("using default channel layout for %d channels", ch);
+            if (!gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
+              GST_WARNING ("failed to add channel layout");
+            }
+          }
+        }
+      } else {
+        /* FIXME: this is pretty useless - we need fixed caps */
+        caps = gst_caps_from_string ("audio/x-raw-float, "
+            "endianness = (int) LITTLE_ENDIAN, " "width = (int) { 32, 64 }");
+      }
+      if (codec_name && strf)
+        *codec_name = g_strdup_printf ("Uncompressed %d-bit IEEE float audio",
+            strf->size);
+      break;
+
     case GST_RIFF_WAVE_FORMAT_IBM_CVSD:
       goto unknown;
 
@@ -818,6 +901,7 @@ gst_riff_create_audio_caps (guint16 codec_id,
           strf->blockalign = strf->av_bps * strf->channels;
         }
       }
+      rate_max = 48000;
       caps = gst_caps_new_simple ("audio/x-alaw", NULL);
       if (codec_name)
         *codec_name = g_strdup ("A-law audio");
@@ -855,6 +939,7 @@ gst_riff_create_audio_caps (guint16 codec_id,
           strf->blockalign = strf->av_bps * strf->channels;
         }
       }
+      rate_max = 48000;
       caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Mu-law audio");
@@ -864,6 +949,7 @@ gst_riff_create_audio_caps (guint16 codec_id,
       goto unknown;
 
     case GST_RIFF_WAVE_FORMAT_DVI_ADPCM:
+      rate_max = 48000;
       caps = gst_caps_new_simple ("audio/x-adpcm",
           "layout", G_TYPE_STRING, "dvi", NULL);
       if (codec_name)
@@ -879,6 +965,8 @@ gst_riff_create_audio_caps (guint16 codec_id,
       break;
 
     case GST_RIFF_WAVE_FORMAT_MPEGL12: /* mp1 or mp2 */
+      rate_min = 16000;
+      rate_max = 48000;
       caps = gst_caps_new_simple ("audio/mpeg",
           "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 2, NULL);
       if (codec_name)
@@ -886,6 +974,8 @@ gst_riff_create_audio_caps (guint16 codec_id,
       break;
 
     case GST_RIFF_WAVE_FORMAT_MPEGL3:  /* mp3 */
+      rate_min = 8000;
+      rate_max = 48000;
       caps = gst_caps_new_simple ("audio/mpeg",
           "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 3, NULL);
       if (codec_name)
@@ -914,6 +1004,8 @@ gst_riff_create_audio_caps (guint16 codec_id,
       caps = gst_caps_new_simple ("audio/x-dts", NULL);
       if (codec_name)
         *codec_name = g_strdup ("DTS audio");
+      /* wavparse is not always able to specify rate/channels for DTS-in-wav */
+      rate_chan = FALSE;
       break;
     case GST_RIFF_WAVE_FORMAT_AAC:
     {
@@ -966,6 +1058,8 @@ gst_riff_create_audio_caps (guint16 codec_id,
       guint32 subformat_guid[4];
       const guint8 *data;
 
+      channels_max = 8;
+
       /* should be at least 22 bytes */
       if (strf_data == NULL || GST_BUFFER_SIZE (strf_data) < 22) {
         GST_WARNING ("WAVE_FORMAT_EXTENSIBLE data size is %d (expected: 22)",
@@ -993,21 +1087,47 @@ gst_riff_create_audio_caps (guint16 codec_id,
           GST_DEBUG ("PCM");
           if (strf != NULL) {
             gint ba = strf->blockalign;
-            gint ws = strf->size;
-            gint depth = ws;
+            gint wd = ba * 8 / strf->channels;
+            gint ws;
 
-            if (valid_bits_per_sample != 0)
-              depth = valid_bits_per_sample;
+            /* in riff, the depth is stored in the size field but it just
+             * means that the _least_ significant bits are cleared. We can
+             * therefore just play the sample as if it had a depth == width */
+            ws = wd;
+
+            /* For reference, use this to get the actual depth:
+             * ws = strf->size;
+             * if (valid_bits_per_sample != 0)
+             *   ws = valid_bits_per_sample; */
 
             caps = gst_caps_new_simple ("audio/x-raw-int",
                 "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
                 "channels", G_TYPE_INT, strf->channels,
-                "width", G_TYPE_INT, (int) (ba * 8 / strf->channels),
-                "depth", G_TYPE_INT, depth,
+                "width", G_TYPE_INT, wd,
+                "depth", G_TYPE_INT, ws,
                 "rate", G_TYPE_INT, strf->rate,
-                "signed", G_TYPE_BOOLEAN, (depth > 8) ? TRUE : FALSE, NULL);
+                "signed", G_TYPE_BOOLEAN, wd != 8, NULL);
 
-            if (!gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
+            /* If channel_mask == 0 and channels <= 2, 4, 6 or 8 let's
+             * assume default layout as some wav files don't have the
+             * channel mask set */
+
+            if (channel_mask == 0) {
+              switch (strf->channels) {
+                case 4:
+                  channel_mask = 0x33;
+                  break;
+                case 6:
+                  channel_mask = 0x3f;
+                  break;
+                case 8:
+                  channel_mask = 0xff;
+                  break;
+              }
+            }
+
+            if ((channel_mask != 0 || strf->channels > 2) &&
+                !gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
               GST_WARNING ("failed to add channel layout");
               gst_caps_unref (caps);
               caps = NULL;
@@ -1020,7 +1140,97 @@ gst_riff_create_audio_caps (guint16 codec_id,
             }
           }
         } else if (subformat_guid[0] == 0x00000003) {
-          GST_DEBUG ("FIXME: handle IEEE float format");
+          GST_DEBUG ("FLOAT");
+          if (strf != NULL) {
+            gint ba = strf->blockalign;
+            gint ws = strf->size;
+            gint wd = ba * 8 / strf->channels;
+
+            if (valid_bits_per_sample != 0)
+              ws = valid_bits_per_sample;
+
+            caps = gst_caps_new_simple ("audio/x-raw-float",
+                "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+                "channels", G_TYPE_INT, strf->channels,
+                "width", G_TYPE_INT, wd, "rate", G_TYPE_INT, strf->rate, NULL);
+
+            /* If channel_mask == 0 and channels <= 2, 4, 6 or 8 let's
+             * assume default layout as some wav files don't have the
+             * channel mask set */
+
+            if (channel_mask == 0) {
+              switch (strf->channels) {
+                case 4:
+                  channel_mask = 0x33;
+                  break;
+                case 6:
+                  channel_mask = 0x3f;
+                  break;
+                case 8:
+                  channel_mask = 0xff;
+                  break;
+              }
+            }
+
+            if ((channel_mask != 0 || strf->channels > 2) &&
+                !gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
+              GST_WARNING ("failed to add channel layout");
+              gst_caps_unref (caps);
+              caps = NULL;
+            }
+            rate_chan = FALSE;
+
+            if (codec_name) {
+              *codec_name =
+                  g_strdup_printf ("Uncompressed %d-bit IEEE float audio",
+                  strf->size);
+            }
+          }
+        } else if (subformat_guid[0] == 00000006) {
+          GST_DEBUG ("ALAW");
+          if (strf != NULL) {
+            if (strf->size != 8) {
+              GST_WARNING ("invalid depth (%d) of alaw audio, overwriting.",
+                  strf->size);
+              strf->size = 8;
+              strf->av_bps = 8;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+            if (strf->av_bps == 0 || strf->blockalign == 0) {
+              GST_WARNING
+                  ("fixing av_bps (%d) and blockalign (%d) of alaw audio",
+                  strf->av_bps, strf->blockalign);
+              strf->av_bps = strf->size;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+          }
+          rate_max = 48000;
+          caps = gst_caps_new_simple ("audio/x-alaw", NULL);
+
+          if (codec_name)
+            *codec_name = g_strdup ("A-law audio");
+        } else if (subformat_guid[0] == 0x00000007) {
+          GST_DEBUG ("MULAW");
+          if (strf != NULL) {
+            if (strf->size != 8) {
+              GST_WARNING ("invalid depth (%d) of mulaw audio, overwriting.",
+                  strf->size);
+              strf->size = 8;
+              strf->av_bps = 8;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+            if (strf->av_bps == 0 || strf->blockalign == 0) {
+              GST_WARNING
+                  ("fixing av_bps (%d) and blockalign (%d) of mulaw audio",
+                  strf->av_bps, strf->blockalign);
+              strf->av_bps = strf->size;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+          }
+          rate_max = 48000;
+          caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
+          if (codec_name)
+            *codec_name = g_strdup ("Mu-law audio");
         } else if (subformat_guid[0] == 0x00000092) {
           GST_DEBUG ("FIXME: handle DOLBY AC3 SPDIF format");
         }
@@ -1066,6 +1276,11 @@ gst_riff_create_audio_caps (guint16 codec_id,
 
   if (strf != NULL) {
     if (rate_chan) {
+      if (strf->channels > channels_max)
+        goto too_many_channels;
+      if (strf->rate < rate_min || strf->rate > rate_max)
+        goto invalid_rate;
+
       gst_caps_set_simple (caps,
           "rate", G_TYPE_INT, strf->rate,
           "channels", G_TYPE_INT, strf->channels, NULL);
@@ -1093,6 +1308,20 @@ gst_riff_create_audio_caps (guint16 codec_id,
   }
 
   return caps;
+
+  /* ERROR */
+too_many_channels:
+  GST_WARNING
+      ("Stream claims to contain %u channels, but format only supports %d",
+      strf->channels, channels_max);
+  gst_caps_unref (caps);
+  return NULL;
+invalid_rate:
+  GST_WARNING
+      ("Stream with sample_rate %u, but format only supports %d .. %d",
+      strf->rate, rate_min, rate_max);
+  gst_caps_unref (caps);
+  return NULL;
 }
 
 GstCaps *
@@ -1220,7 +1449,9 @@ gst_riff_create_audio_template_caps (void)
     GST_RIFF_WAVE_FORMAT_WMAV1,
     GST_RIFF_WAVE_FORMAT_WMAV2,
     GST_RIFF_WAVE_FORMAT_WMAV3,
-    GST_RIFF_WAVE_FORMAT_SONY_ATRAC3
+    GST_RIFF_WAVE_FORMAT_SONY_ATRAC3,
+    GST_RIFF_WAVE_FORMAT_IEEE_FLOAT,
+    GST_RIFF_WAVE_FORMAT_VOXWARE
         /* FILL ME */
   };
   guint i;
