@@ -23,7 +23,7 @@
 /**
  * SECTION:gstrtcpbuffer
  * @short_description: Helper methods for dealing with RTCP buffers
- * @see_also: #GstBaseRTPPayload, #GstBaseRTPDepayload, #gstrtpbuffer
+ * @see_also: gstbasertppayload, gstbasertpdepayload
  *
  * Note: The API in this module is not yet declared stable.
  *
@@ -41,9 +41,9 @@
  * </para>
  * </refsect2>
  *
- * Last reviewed on 2007-03-26 (0.10.13)
- *
  * Since: 0.10.13
+ *
+ * Last reviewed on 2007-03-26 (0.10.13)
  */
 
 #include <string.h>
@@ -111,7 +111,7 @@ gboolean
 gst_rtcp_buffer_validate_data (guint8 * data, guint len)
 {
   guint16 header_mask;
-  guint header_len;
+  guint16 header_len;
   guint8 version;
   guint data_len;
   gboolean padding;
@@ -163,7 +163,7 @@ gst_rtcp_buffer_validate_data (guint8 * data, guint len)
       goto wrong_length;
 
     /* get padding */
-    pad_bytes = data[data_len - 1];
+    pad_bytes = data[len - 1];
     if (data_len != pad_bytes)
       goto wrong_padding;
   }
@@ -246,9 +246,6 @@ gst_rtcp_buffer_new (guint mtu)
  *
  * Finish @buffer after being constructured. This function is usually called
  * after gst_rtcp_buffer_new() and after adding the RTCP items to the new buffer. 
- *
- * The function adjusts the size of @buffer with the total length of all the
- * added packets.
  */
 void
 gst_rtcp_buffer_end (GstBuffer * buffer)
@@ -449,12 +446,6 @@ gst_rtcp_buffer_add_packet (GstBuffer * buffer, GstRTCPType type,
     case GST_RTCP_TYPE_APP:
       len = 12;
       break;
-    case GST_RTCP_TYPE_RTPFB:
-      len = 12;
-      break;
-    case GST_RTCP_TYPE_PSFB:
-      len = 12;
-      break;
     default:
       goto unknown_type;
   }
@@ -492,34 +483,17 @@ no_space:
  * gst_rtcp_packet_remove:
  * @packet: a #GstRTCPPacket
  *
- * Removes the packet pointed to by @packet and moves pointer to the next one
+ * Removes the packet pointed to by @packet.
  *
- * Returns: TRUE if @packet is pointing to a valid packet after calling this
- * function.
+ * Note: Not implemented.
  */
-gboolean
+void
 gst_rtcp_packet_remove (GstRTCPPacket * packet)
 {
-  gboolean ret = FALSE;
-  guint offset = 0;
+  g_return_if_fail (packet != NULL);
+  g_return_if_fail (packet->type != GST_RTCP_TYPE_INVALID);
 
-  g_return_val_if_fail (packet != NULL, FALSE);
-  g_return_val_if_fail (packet->type != GST_RTCP_TYPE_INVALID, FALSE);
-
-  /* The next packet starts at offset + length + 4 (the header) */
-  offset = packet->offset + (packet->length << 2) + 4;
-
-  /* Overwrite this packet with the rest of the data */
-  memmove (GST_BUFFER_DATA (packet->buffer) + packet->offset,
-      GST_BUFFER_DATA (packet->buffer) + offset,
-      GST_BUFFER_SIZE (packet->buffer) - offset);
-
-  /* try to read next header */
-  ret = read_packet_header (packet);
-  if (!ret)
-    packet->type = GST_RTCP_TYPE_INVALID;
-
-  return ret;
+  g_warning ("not implemented");
 }
 
 /**
@@ -545,13 +519,14 @@ gst_rtcp_packet_get_padding (GstRTCPPacket * packet)
  *
  * Get the packet type of the packet pointed to by @packet.
  *
- * Returns: The packet type or GST_RTCP_TYPE_INVALID when @packet is not
- * pointing to a valid packet.
+ * Returns: The packet type.
  */
 GstRTCPType
 gst_rtcp_packet_get_type (GstRTCPPacket * packet)
 {
   g_return_val_if_fail (packet != NULL, GST_RTCP_TYPE_INVALID);
+  g_return_val_if_fail (packet->type != GST_RTCP_TYPE_INVALID,
+      GST_RTCP_TYPE_INVALID);
 
   return packet->type;
 }
@@ -599,7 +574,7 @@ gst_rtcp_packet_get_length (GstRTCPPacket * packet)
  * @ntptime: result NTP time
  * @rtptime: result RTP time
  * @packet_count: result packet count
- * @octet_count: result octet count
+ * @octet_count: result octect count
  *
  * Parse the SR sender info and store the values.
  */
@@ -641,7 +616,7 @@ gst_rtcp_packet_sr_get_sender_info (GstRTCPPacket * packet, guint32 * ssrc,
  * @ntptime: the NTP time
  * @rtptime: the RTP time
  * @packet_count: the packet count
- * @octet_count: the octet count
+ * @octet_count: the octect count
  *
  * Set the given values in the SR packet @packet.
  */
@@ -783,6 +758,7 @@ gst_rtcp_packet_get_rb (GstRTCPPacket * packet, guint nth, guint32 * ssrc,
     *ssrc = GST_READ_UINT32_BE (data);
   data += 4;
   tmp = GST_READ_UINT32_BE (data);
+  data += 4;
   if (fractionlost)
     *fractionlost = (tmp >> 24);
   if (packetslost) {
@@ -997,7 +973,7 @@ gst_rtcp_packet_sdes_next_item (GstRTCPPacket * packet)
   while (offset < len) {
     if (data[offset] == 0) {
       /* end of list, round to next 32-bit word */
-      offset = (offset + 4) & ~3;
+      offset = (offset + 3) & ~3;
       break;
     }
     offset += data[offset + 1] + 2;
@@ -1131,13 +1107,7 @@ gst_rtcp_packet_sdes_next_entry (GstRTCPPacket * packet)
  * @len: result length of the entry data
  * @data: result entry data
  *
- * Get the data of the current SDES item entry. @type (when not NULL) will
- * contain the type of the entry. @data (when not NULL) will point to @len
- * bytes.
- *
- * When @type refers to a text item, @data will point to a UTF8 string. Note
- * that this UTF8 string is NOT null-terminated. Use
- * gst_rtcp_packet_sdes_copy_entry() to get a null-terminated copy of the entry.
+ * Get the data of the current SDES item entry.
  *
  * Returns: %TRUE if there was valid data.
  */
@@ -1168,41 +1138,7 @@ gst_rtcp_packet_sdes_get_entry (GstRTCPPacket * packet,
   if (len)
     *len = bdata[offset + 1];
   if (data)
-    *data = &bdata[offset + 2];
-
-  return TRUE;
-}
-
-/**
- * gst_rtcp_packet_sdes_copy_entry:
- * @packet: a valid SDES #GstRTCPPacket
- * @type: result of the entry type
- * @len: result length of the entry data
- * @data: result entry data
- *
- * This function is like gst_rtcp_packet_sdes_get_entry() but it returns a
- * null-terminated copy of the data instead. use g_free() after usage.
- *
- * Returns: %TRUE if there was valid data.
- */
-gboolean
-gst_rtcp_packet_sdes_copy_entry (GstRTCPPacket * packet,
-    GstRTCPSDESType * type, guint8 * len, guint8 ** data)
-{
-  guint8 *tdata;
-  guint8 tlen;
-
-  g_return_val_if_fail (packet != NULL, FALSE);
-  g_return_val_if_fail (packet->type == GST_RTCP_TYPE_SDES, FALSE);
-  g_return_val_if_fail (GST_IS_BUFFER (packet->buffer), FALSE);
-
-  if (!gst_rtcp_packet_sdes_get_entry (packet, type, &tlen, &tdata))
-    return FALSE;
-
-  if (len)
-    *len = tlen;
-  if (data)
-    *data = (guint8 *) g_strndup ((gchar *) tdata, tlen);
+    *data = g_memdup (&bdata[offset + 2], bdata[offset + 1]);
 
   return TRUE;
 }
@@ -1555,7 +1491,7 @@ gst_rtcp_packet_bye_get_reason (GstRTCPPacket * packet)
   guint8 len;
 
   g_return_val_if_fail (packet != NULL, NULL);
-  g_return_val_if_fail (packet->type == GST_RTCP_TYPE_BYE, NULL);
+  g_return_val_if_fail (packet->type == GST_RTCP_TYPE_BYE, 0);
   g_return_val_if_fail (GST_IS_BUFFER (packet->buffer), NULL);
 
   roffset = get_reason_offset (packet);
@@ -1587,6 +1523,8 @@ gst_rtcp_packet_bye_get_reason (GstRTCPPacket * packet)
  * Set the reason string to @reason in @packet.
  *
  * Returns: TRUE if the string could be set.
+ *
+ * Note: Not implemented.
  */
 gboolean
 gst_rtcp_packet_bye_set_reason (GstRTCPPacket * packet, const gchar * reason)
@@ -1640,392 +1578,3 @@ no_space:
     return FALSE;
   }
 }
-
-/**
- * gst_rtcp_packet_fb_get_sender_ssrc:
- * @packet: a valid RTPFB or PSFB #GstRTCPPacket
- *
- * Get the sender SSRC field of the RTPFB or PSFB @packet.
- *
- * Returns: the sender SSRC.
- *
- * Since: 0.10.23
- */
-guint32
-gst_rtcp_packet_fb_get_sender_ssrc (GstRTCPPacket * packet)
-{
-  guint8 *data;
-  guint32 ssrc;
-
-  g_return_val_if_fail (packet != NULL, 0);
-  g_return_val_if_fail ((packet->type == GST_RTCP_TYPE_RTPFB ||
-          packet->type == GST_RTCP_TYPE_PSFB), 0);
-  g_return_val_if_fail (GST_IS_BUFFER (packet->buffer), 0);
-
-  data = GST_BUFFER_DATA (packet->buffer);
-
-  /* skip header */
-  data += packet->offset + 4;
-  ssrc = GST_READ_UINT32_BE (data);
-
-  return ssrc;
-}
-
-/**
- * gst_rtcp_packet_fb_set_sender_ssrc:
- * @packet: a valid RTPFB or PSFB #GstRTCPPacket
- * @ssrc: a sender SSRC
- *
- * Set the sender SSRC field of the RTPFB or PSFB @packet.
- *
- * Since: 0.10.23
- */
-void
-gst_rtcp_packet_fb_set_sender_ssrc (GstRTCPPacket * packet, guint32 ssrc)
-{
-  guint8 *data;
-
-  g_return_if_fail (packet != NULL);
-  g_return_if_fail (packet->type == GST_RTCP_TYPE_RTPFB ||
-      packet->type == GST_RTCP_TYPE_PSFB);
-  g_return_if_fail (GST_IS_BUFFER (packet->buffer));
-
-  data = GST_BUFFER_DATA (packet->buffer);
-
-  /* skip header */
-  data += packet->offset + 4;
-  GST_WRITE_UINT32_BE (data, ssrc);
-}
-
-/**
- * gst_rtcp_packet_fb_get_media_ssrc:
- * @packet: a valid RTPFB or PSFB #GstRTCPPacket
- *
- * Get the media SSRC field of the RTPFB or PSFB @packet.
- *
- * Returns: the media SSRC.
- *
- * Since: 0.10.23
- */
-guint32
-gst_rtcp_packet_fb_get_media_ssrc (GstRTCPPacket * packet)
-{
-  guint8 *data;
-  guint32 ssrc;
-
-  g_return_val_if_fail (packet != NULL, 0);
-  g_return_val_if_fail ((packet->type == GST_RTCP_TYPE_RTPFB ||
-          packet->type == GST_RTCP_TYPE_PSFB), 0);
-  g_return_val_if_fail (GST_IS_BUFFER (packet->buffer), 0);
-
-  data = GST_BUFFER_DATA (packet->buffer);
-
-  /* skip header and sender ssrc */
-  data += packet->offset + 8;
-  ssrc = GST_READ_UINT32_BE (data);
-
-  return ssrc;
-}
-
-/**
- * gst_rtcp_packet_fb_set_media_ssrc:
- * @packet: a valid RTPFB or PSFB #GstRTCPPacket
- * @ssrc: a media SSRC
- *
- * Set the media SSRC field of the RTPFB or PSFB @packet.
- *
- * Since: 0.10.23
- */
-void
-gst_rtcp_packet_fb_set_media_ssrc (GstRTCPPacket * packet, guint32 ssrc)
-{
-  guint8 *data;
-
-  g_return_if_fail (packet != NULL);
-  g_return_if_fail (packet->type == GST_RTCP_TYPE_RTPFB ||
-      packet->type == GST_RTCP_TYPE_PSFB);
-  g_return_if_fail (GST_IS_BUFFER (packet->buffer));
-
-  data = GST_BUFFER_DATA (packet->buffer);
-
-  /* skip header and sender ssrc */
-  data += packet->offset + 8;
-  GST_WRITE_UINT32_BE (data, ssrc);
-}
-
-/**
- * gst_rtcp_packet_fb_get_type:
- * @packet: a valid RTPFB or PSFB #GstRTCPPacket
- *
- * Get the feedback message type of the FB @packet.
- *
- * Returns: The feedback message type.
- *
- * Since: 0.10.23
- */
-GstRTCPFBType
-gst_rtcp_packet_fb_get_type (GstRTCPPacket * packet)
-{
-  g_return_val_if_fail (packet != NULL, GST_RTCP_FB_TYPE_INVALID);
-  g_return_val_if_fail (packet->type == GST_RTCP_TYPE_RTPFB ||
-      packet->type == GST_RTCP_TYPE_PSFB, GST_RTCP_FB_TYPE_INVALID);
-
-  return packet->count;
-}
-
-/**
- * gst_rtcp_packet_fb_set_type:
- * @packet: a valid RTPFB or PSFB #GstRTCPPacket
- * @type: the #GstRTCPFBType to set
- *
- * Set the feedback message type of the FB @packet.
- *
- * Since: 0.10.23
- */
-void
-gst_rtcp_packet_fb_set_type (GstRTCPPacket * packet, GstRTCPFBType type)
-{
-  guint8 *data;
-
-  g_return_if_fail (packet != NULL);
-  g_return_if_fail (packet->type == GST_RTCP_TYPE_RTPFB ||
-      packet->type == GST_RTCP_TYPE_PSFB);
-  g_return_if_fail (GST_IS_BUFFER (packet->buffer));
-
-  data = GST_BUFFER_DATA (packet->buffer);
-
-  data[packet->offset] = (data[packet->offset] & 0xe0) | type;
-  packet->count = type;
-}
-
-/**
- * gst_rtcp_ntp_to_unix:
- * @ntptime: an NTP timestamp
- *
- * Converts an NTP time to UNIX nanoseconds. @ntptime can typically be
- * the NTP time of an SR RTCP message and contains, in the upper 32 bits, the
- * number of seconds since 1900 and, in the lower 32 bits, the fractional
- * seconds. The resulting value will be the number of nanoseconds since 1970.
- *
- * Returns: the UNIX time for @ntptime in nanoseconds.
- */
-guint64
-gst_rtcp_ntp_to_unix (guint64 ntptime)
-{
-  guint64 unixtime;
-
-  /* conversion from NTP timestamp (seconds since 1900) to seconds since
-   * 1970. */
-  unixtime = ntptime - (G_GUINT64_CONSTANT (2208988800) << 32);
-  /* conversion to nanoseconds */
-  unixtime =
-      gst_util_uint64_scale (unixtime, GST_SECOND,
-      (G_GINT64_CONSTANT (1) << 32));
-
-  return unixtime;
-}
-
-/**
- * gst_rtcp_unix_to_ntp:
- * @unixtime: an UNIX timestamp in nanoseconds
- *
- * Converts a UNIX timestamp in nanoseconds to an NTP time. The caller should
- * pass a value with nanoseconds since 1970. The NTP time will, in the upper
- * 32 bits, contain the number of seconds since 1900 and, in the lower 32
- * bits, the fractional seconds. The resulting value can be used as an ntptime
- * for constructing SR RTCP packets.
- *
- * Returns: the NTP time for @unixtime.
- */
-guint64
-gst_rtcp_unix_to_ntp (guint64 unixtime)
-{
-  guint64 ntptime;
-
-  /* convert clock time to NTP time. upper 32 bits should contain the seconds
-   * and the lower 32 bits, the fractions of a second. */
-  ntptime =
-      gst_util_uint64_scale (unixtime, (G_GINT64_CONSTANT (1) << 32),
-      GST_SECOND);
-  /* conversion from UNIX timestamp (seconds since 1970) to NTP (seconds
-   * since 1900). */
-  ntptime += (G_GUINT64_CONSTANT (2208988800) << 32);
-
-  return ntptime;
-}
-
-/**
- * gst_rtcp_sdes_type_to_name:
- * @type: a #GstRTCPSDESType
- *
- * Converts @type to the string equivalent. The string is typically used as a
- * key in a #GstStructure containing SDES items.
- *
- * Returns: the string equivalent of @type
- *
- * Since: 0.10.26
- */
-const gchar *
-gst_rtcp_sdes_type_to_name (GstRTCPSDESType type)
-{
-  const gchar *result;
-
-  switch (type) {
-    case GST_RTCP_SDES_CNAME:
-      result = "cname";
-      break;
-    case GST_RTCP_SDES_NAME:
-      result = "name";
-      break;
-    case GST_RTCP_SDES_EMAIL:
-      result = "email";
-      break;
-    case GST_RTCP_SDES_PHONE:
-      result = "phone";
-      break;
-    case GST_RTCP_SDES_LOC:
-      result = "location";
-      break;
-    case GST_RTCP_SDES_TOOL:
-      result = "tool";
-      break;
-    case GST_RTCP_SDES_NOTE:
-      result = "note";
-      break;
-    case GST_RTCP_SDES_PRIV:
-      result = "priv";
-      break;
-    default:
-      result = NULL;
-      break;
-  }
-  return result;
-}
-
-/**
- * gst_rtcp_sdes_name_to_type:
- * @name: a SDES name
- *
- * Convert @name into a @GstRTCPSDESType. @name is typically a key in a
- * #GstStructure containing SDES items.
- *
- * Returns: the #GstRTCPSDESType for @name or #GST_RTCP_SDES_PRIV when @name
- * is a private sdes item.
- *
- * Since: 0.10.26
- */
-GstRTCPSDESType
-gst_rtcp_sdes_name_to_type (const gchar * name)
-{
-  if (name == NULL || strlen (name) == 0)
-    return GST_RTCP_SDES_INVALID;
-
-  if (strcmp ("cname", name) == 0)
-    return GST_RTCP_SDES_CNAME;
-
-  if (strcmp ("name", name) == 0)
-    return GST_RTCP_SDES_NAME;
-
-  if (strcmp ("email", name) == 0)
-    return GST_RTCP_SDES_EMAIL;
-
-  if (strcmp ("phone", name) == 0)
-    return GST_RTCP_SDES_PHONE;
-
-  if (strcmp ("location", name) == 0)
-    return GST_RTCP_SDES_LOC;
-
-  if (strcmp ("tool", name) == 0)
-    return GST_RTCP_SDES_TOOL;
-
-  if (strcmp ("note", name) == 0)
-    return GST_RTCP_SDES_NOTE;
-
-  return GST_RTCP_SDES_PRIV;
-}
-
-/**
- * gst_rtcp_packet_fb_get_fci_length:
- * @packet: a valid RTPFB or PSFB #GstRTCPPacket
- *
- * Get the length of the Feedback Control Information attached to a
- * RTPFB or PSFB @packet.
- *
- * Returns: The length of the FCI in 32-bit words.
- *
- * Since: 0.10.31
- */
-guint16
-gst_rtcp_packet_fb_get_fci_length (GstRTCPPacket * packet)
-{
-  guint8 *data;
-
-  g_return_val_if_fail (packet != NULL, 0);
-  g_return_val_if_fail (packet->type == GST_RTCP_TYPE_RTPFB ||
-      packet->type == GST_RTCP_TYPE_PSFB, 0);
-  g_return_val_if_fail (GST_IS_BUFFER (packet->buffer), 0);
-
-  data = GST_BUFFER_DATA (packet->buffer) + packet->offset + 2;
-
-  return GST_READ_UINT16_BE (data) - 2;
-}
-
-/**
- * gst_rtcp_packet_fb_set_fci_length:
- * @packet: a valid RTPFB or PSFB #GstRTCPPacket
- * @wordlen: Length of the FCI in 32-bit words
- *
- * Set the length of the Feedback Control Information attached to a
- * RTPFB or PSFB @packet.
- *
- * Returns: %TRUE if there was enough space in the packet to add this much FCI
- *
- * Since: 0.10.31
- */
-gboolean
-gst_rtcp_packet_fb_set_fci_length (GstRTCPPacket * packet, guint16 wordlen)
-{
-  guint8 *data;
-
-  g_return_val_if_fail (packet != NULL, FALSE);
-  g_return_val_if_fail (packet->type == GST_RTCP_TYPE_RTPFB ||
-      packet->type == GST_RTCP_TYPE_PSFB, FALSE);
-  g_return_val_if_fail (GST_IS_BUFFER (packet->buffer), FALSE);
-
-  if (GST_BUFFER_SIZE (packet->buffer) < packet->offset + ((wordlen + 3) * 4))
-    return FALSE;
-
-  data = GST_BUFFER_DATA (packet->buffer) + packet->offset + 2;
-  wordlen += 2;
-  GST_WRITE_UINT16_BE (data, wordlen);
-
-  return TRUE;
-}
-
-/**
- * gst_rtcp_packet_fb_get_fci:
- * @packet: a valid RTPFB or PSFB #GstRTCPPacket
- *
- * Get the Feedback Control Information attached to a RTPFB or PSFB @packet.
- *
- * Returns: a pointer to the FCI
- *
- * Since: 0.10.31
- */
-guint8 *
-gst_rtcp_packet_fb_get_fci (GstRTCPPacket * packet)
-{
-  guint8 *data;
-
-  g_return_val_if_fail (packet != NULL, NULL);
-  g_return_val_if_fail (packet->type == GST_RTCP_TYPE_RTPFB ||
-      packet->type == GST_RTCP_TYPE_PSFB, NULL);
-  g_return_val_if_fail (GST_IS_BUFFER (packet->buffer), NULL);
-
-  data = GST_BUFFER_DATA (packet->buffer) + packet->offset;
-
-  if (GST_READ_UINT16_BE (data + 2) <= 2)
-    return NULL;
-
-  return data + 12;
-}
