@@ -849,3 +849,55 @@ gst_rtp_buffer_get_payload (GstBuffer * buffer)
   return GST_BUFFER_DATA (buffer) + GST_RTP_HEADER_LEN
       + GST_RTP_HEADER_CSRC_SIZE (buffer);
 }
+
+/**
+ * gst_rtp_buffer_default_clock_rate:
+ * @payload_type: the static payload type
+ *
+ * Get the default clock-rate for the static payload type @payload_type.
+ *
+ * Returns: the default clock rate or -1 if the payload type is not static or
+ * the clock-rate is undefined.
+ *
+ * Since: 0.10.13
+ */
+guint32
+gst_rtp_buffer_default_clock_rate (guint8 payload_type)
+{
+  switch (payload_type) {
+    case GST_RTP_PAYLOAD_PCMU:
+    case GST_RTP_PAYLOAD_GSM:
+    case GST_RTP_PAYLOAD_G723:
+    case GST_RTP_PAYLOAD_DVI4_8000:
+    case GST_RTP_PAYLOAD_LPC:
+    case GST_RTP_PAYLOAD_PCMA:
+    case GST_RTP_PAYLOAD_G722:
+    case GST_RTP_PAYLOAD_G729:
+    case GST_RTP_PAYLOAD_QCELP:
+    case GST_RTP_PAYLOAD_CN:
+    case GST_RTP_PAYLOAD_G728:
+      return 8000;
+    case GST_RTP_PAYLOAD_DVI4_11025:
+      return 11025;
+    case GST_RTP_PAYLOAD_DVI4_16000:
+      return 16000;
+    case GST_RTP_PAYLOAD_DVI4_22050:
+      return 22050;
+    case GST_RTP_PAYLOAD_L16_STEREO:
+    case GST_RTP_PAYLOAD_L16_MONO:
+      return 44100;
+    case GST_RTP_PAYLOAD_MPA:
+    case GST_RTP_PAYLOAD_CELLB:
+    case GST_RTP_PAYLOAD_JPEG:
+    case GST_RTP_PAYLOAD_NV:
+    case GST_RTP_PAYLOAD_H261:
+    case GST_RTP_PAYLOAD_MPV:
+    case GST_RTP_PAYLOAD_MP2T:
+    case GST_RTP_PAYLOAD_H263:
+      return 90000;
+    case GST_RTP_PAYLOAD_1016:
+    case GST_RTP_PAYLOAD_G721:
+    default:
+      return -1;
+  }
+}
