@@ -1,8 +1,5 @@
 /* GStreamer
- * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
- *                    2000 Wim Taymans <wtay@chello.be>
- *
- * gstsinesrc.h: 
+ * Copyright (C) 2005 Stefan Kost <ensonic@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,7 +17,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-
 #ifndef __GST_AUDIO_TEST_SRC_H__
 #define __GST_AUDIO_TEST_SRC_H__
 
@@ -39,7 +35,7 @@ G_BEGIN_DECLS
   (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_AUDIO_TEST_SRC,GstAudioTestSrcClass))
 #define GST_IS_AUDIO_TEST_SRC(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_AUDIO_TEST_SRC))
-#define GST_IS_AUDIO_TEST_SRC_CLASS(obj) \
+#define GST_IS_AUDIO_TEST_SRC_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_AUDIO_TEST_SRC))
 
 /**
@@ -120,8 +116,7 @@ struct _GstAudioTestSrcClass {
   GstBaseSrcClass parent_class;
 };
 
-GType gst_audio_test_src_get_type(void);
-gboolean gst_audio_test_src_factory_init (GstElementFactory *factory);
+GType gst_audio_test_src_get_type (void);
 
 G_END_DECLS
 
