@@ -27,6 +27,8 @@
 #include "gststreamselector.h"
 #include "gstplay-marshal.h"
 
+#include <gst/pbutils/pbutils.h>
+
 GST_DEBUG_CATEGORY_STATIC (gst_play_base_bin_debug);
 #define GST_CAT_DEFAULT gst_play_base_bin_debug
 
@@ -67,12 +69,14 @@ static void gst_play_base_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * spec);
 static void gst_play_base_bin_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * spec);
+static void gst_play_base_bin_handle_message_func (GstBin * bin,
+    GstMessage * msg);
 
 static GstStateChangeReturn gst_play_base_bin_change_state (GstElement *
     element, GstStateChange transition);
 
-const GList *gst_play_base_bin_get_streaminfo (GstPlayBaseBin * play_base_bin);
-const GValueArray *gst_play_base_bin_get_streaminfo_value_array (GstPlayBaseBin
+static const GList *gst_play_base_bin_get_streaminfo (GstPlayBaseBin * bin);
+static GValueArray *gst_play_base_bin_get_streaminfo_value_array (GstPlayBaseBin
     * play_base_bin);
 static void preroll_remove_overrun (GstElement * element,
     GstPlayBaseBin * play_base_bin);
@@ -194,6 +198,9 @@ gst_play_base_bin_class_init (GstPlayBaseBinClass * klass)
   gobject_klass->dispose = GST_DEBUG_FUNCPTR (gst_play_base_bin_dispose);
   gobject_klass->finalize = GST_DEBUG_FUNCPTR (gst_play_base_bin_finalize);
 
+  gstbin_klass->handle_message =
+      GST_DEBUG_FUNCPTR (gst_play_base_bin_handle_message_func);
+
   gstelement_klass->change_state =
       GST_DEBUG_FUNCPTR (gst_play_base_bin_change_state);
 }
@@ -489,6 +496,7 @@ group_is_muted (GstPlayBaseGroup * group)
 static inline void
 fill_buffer (GstPlayBaseBin * play_base_bin, gint percent)
 {
+  GST_DEBUG_OBJECT (play_base_bin, "buffering %d", percent);
   gst_element_post_message (GST_ELEMENT_CAST (play_base_bin),
       gst_message_new_buffering (GST_OBJECT_CAST (play_base_bin), percent));
 }
@@ -521,10 +529,12 @@ check_queue (GstPad * pad, GstBuffer * data, gpointer user_data)
   GstPlayBaseBin *play_base_bin = g_object_get_data (G_OBJECT (queue), "pbb");
   guint64 level = 0;
 
-  GST_DEBUG ("check queue triggered");
+  GST_DEBUG_OBJECT (queue, "check queue triggered");
 
   g_object_get (G_OBJECT (queue), "current-level-time", &level, NULL);
-  GST_DEBUG ("Queue size: %" GST_TIME_FORMAT, GST_TIME_ARGS (level));
+  GST_DEBUG_OBJECT (play_base_bin, "Queue size: %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (level));
+
   if (play_base_bin->queue_threshold > 0) {
     level = level * 99 / play_base_bin->queue_threshold;
     if (level > 99)
@@ -587,25 +597,6 @@ queue_deadlock_check (GstElement * queue, GstPlayBaseBin * play_base_bin)
   }
 }
 
-/* this signal will be fired when one of the queues with raw
- * data is filled. This means that the group building stage is over 
- * and playback of the new queued group should start. This is a rather unusual
- * situation because normally the group is commited when the "no_more_pads"
- * signal is fired.
- */
-static void
-queue_overrun (GstElement * queue, GstPlayBaseBin * play_base_bin)
-{
-  GST_DEBUG_OBJECT (play_base_bin, "queue %s overrun",
-      GST_ELEMENT_NAME (queue));
-
-  preroll_remove_overrun (queue, play_base_bin);
-
-  group_commit (play_base_bin, FALSE,
-      GST_OBJECT_PARENT (GST_OBJECT_CAST (queue)) ==
-      GST_OBJECT_CAST (play_base_bin->subtitle));
-}
-
 /* Used for time-based buffering in streaming mode and is called when a queue
  * emits the running signal. This means that the high watermark threshold is
  * reached and the buffering is completed. */
@@ -657,6 +648,28 @@ queue_threshold_reached (GstElement * queue, GstPlayBaseBin * play_base_bin)
   fill_buffer (play_base_bin, 100);
 }
 
+/* this signal will be fired when one of the queues with raw
+ * data is filled. This means that the group building stage is over 
+ * and playback of the new queued group should start. This is a rather unusual
+ * situation because normally the group is commited when the "no_more_pads"
+ * signal is fired.
+ */
+static void
+queue_overrun (GstElement * queue, GstPlayBaseBin * play_base_bin)
+{
+  GST_DEBUG_OBJECT (play_base_bin, "queue %s overrun",
+      GST_ELEMENT_NAME (queue));
+
+  preroll_remove_overrun (queue, play_base_bin);
+
+  group_commit (play_base_bin, FALSE,
+      GST_OBJECT_PARENT (GST_OBJECT_CAST (queue)) ==
+      GST_OBJECT_CAST (play_base_bin->subtitle));
+
+  /* notify end of buffering */
+  queue_threshold_reached (queue, play_base_bin);
+}
+
 /* this signal is only added when in streaming mode to catch underruns
  */
 static void
@@ -671,7 +684,7 @@ queue_out_of_data (GstElement * queue, GstPlayBaseBin * play_base_bin)
    * This signal could never be called because the queue max-size limits are set
    * too low. We take care of this possible deadlock in the the overrun signal
    * handler. */
-  g_signal_connect (G_OBJECT (queue), "running",
+  g_signal_connect (G_OBJECT (queue), "pushing",
       G_CALLBACK (queue_threshold_reached), play_base_bin);
   GST_DEBUG_OBJECT (play_base_bin,
       "setting min threshold time to %" G_GUINT64_FORMAT,
@@ -679,7 +692,7 @@ queue_out_of_data (GstElement * queue, GstPlayBaseBin * play_base_bin)
   g_object_set (queue, "min-threshold-time",
       (guint64) play_base_bin->queue_threshold, NULL);
 
-  /* re-connect probe, this will five feedback about the percentage that we
+  /* re-connect probe, this will fire feedback about the percentage that we
    * buffered and is posted in the BUFFERING message. */
   if (!g_object_get_data (G_OBJECT (queue), "probe")) {
     GstPad *sinkpad;
@@ -905,12 +918,55 @@ add_stream (GstPlayBaseGroup * group, GstStreamInfo * info)
   g_value_init (&v, G_TYPE_OBJECT);
   g_value_set_object (&v, info);
   g_value_array_append (group->streaminfo_value_array, &v);
+  g_value_unset (&v);
   group->streaminfo = g_list_append (group->streaminfo, info);
   if (info->type > 0 && info->type <= NUM_TYPES) {
     group->type[info->type - 1].npads++;
   }
 }
 
+static gboolean
+string_arr_has_str (const gchar * values[], const gchar * value)
+{
+  if (values && value) {
+    while (*values != NULL) {
+      if (strcmp (value, *values) == 0)
+        return TRUE;
+      ++values;
+    }
+  }
+  return FALSE;
+}
+
+/* mime types we are not handling on purpose right now, don't post a
+ * missing-plugin message for these */
+static const gchar *blacklisted_mimes[] = {
+  "video/x-dvd-subpicture", NULL
+};
+
+#define IS_BLACKLISTED_MIME(type) (string_arr_has_str(blacklisted_mimes,type))
+
+static void
+gst_play_base_bin_handle_message_func (GstBin * bin, GstMessage * msg)
+{
+  if (gst_is_missing_plugin_message (msg)) {
+    gchar *detail;
+    guint i;
+
+    detail = gst_missing_plugin_message_get_installer_detail (msg);
+    for (i = 0; detail != NULL && blacklisted_mimes[i] != NULL; ++i) {
+      if (strstr (detail, "|decoder-") && strstr (detail, blacklisted_mimes[i])) {
+        GST_LOG_OBJECT (bin, "suppressing message %" GST_PTR_FORMAT, msg);
+        gst_message_unref (msg);
+        g_free (detail);
+        return;
+      }
+    }
+    g_free (detail);
+  }
+  GST_BIN_CLASS (parent_class)->handle_message (bin, msg);
+}
+
 /*
  * signal fired when an unknown stream is found. We create a new
  * UNKNOWN streaminfo object.
@@ -919,14 +975,24 @@ static void
 unknown_type (GstElement * element, GstPad * pad, GstCaps * caps,
     GstPlayBaseBin * play_base_bin)
 {
-  gchar *capsstr;
+  const gchar *type_name;
   GstStreamInfo *info;
   GstPlayBaseGroup *group;
 
-  capsstr = gst_caps_to_string (caps);
-  GST_DEBUG_OBJECT (play_base_bin, "don't know how to handle %s", capsstr);
-  /* FIXME, g_message() ? */
-  g_message ("don't know how to handle %s", capsstr);
+  type_name = gst_structure_get_name (gst_caps_get_structure (caps, 0));
+  if (type_name && !IS_BLACKLISTED_MIME (type_name)) {
+    gchar *capsstr;
+
+    capsstr = gst_caps_to_string (caps);
+    GST_DEBUG_OBJECT (play_base_bin, "don't know how to handle %s", capsstr);
+    /* FIXME, g_message() ? */
+    g_message ("don't know how to handle %s", capsstr);
+    g_free (capsstr);
+  } else {
+    /* don't spew stuff to the terminal or send message if it's blacklisted */
+    GST_DEBUG_OBJECT (play_base_bin, "media type %s not handled on purpose, "
+        "not posting a missing-plugin message on the bus", type_name);
+  }
 
   GROUP_LOCK (play_base_bin);
 
@@ -939,8 +1005,6 @@ unknown_type (GstElement * element, GstPad * pad, GstCaps * caps,
   add_stream (group, info);
 
   GROUP_UNLOCK (play_base_bin);
-
-  g_free (capsstr);
 }
 
 /* add a streaminfo that indicates that the stream is handled by the
@@ -990,6 +1054,10 @@ no_more_pads_full (GstElement * element, gboolean subs,
   g_object_set_data (G_OBJECT (element), "pending", NULL);
 
   play_base_bin->pending--;
+
+  GST_DEBUG_OBJECT (element, "remove pending, now %d pending",
+      play_base_bin->pending);
+
   if (play_base_bin->pending == 0) {
     /* we can commit this group for playback now */
     group_commit (play_base_bin, play_base_bin->is_stream, subs);
@@ -1187,11 +1255,12 @@ new_decoded_pad_full (GstElement * element, GstPad * pad, gboolean last,
   const gchar *mimetype;
   GstCaps *caps;
   GstStreamInfo *info;
-  GstStreamType type;
+  GstStreamType type = GST_STREAM_TYPE_UNKNOWN;
   GstPad *sinkpad;
   GstPlayBaseGroup *group;
   guint sig;
   GstObject *parent;
+  gboolean first_pad;
 
   GST_DEBUG ("play base: new decoded pad. Last: %d", last);
 
@@ -1219,18 +1288,32 @@ new_decoded_pad_full (GstElement * element, GstPad * pad, gboolean last,
     type = GST_STREAM_TYPE_VIDEO;
   } else if (g_str_has_prefix (mimetype, "text/")) {
     type = GST_STREAM_TYPE_TEXT;
-  } else {
-    type = GST_STREAM_TYPE_UNKNOWN;
   }
   gst_object_unref (parent);
 
   info = gst_stream_info_new (GST_OBJECT_CAST (pad), type, NULL, caps);
-  if (type > 0 && type <= NUM_TYPES) {
-    /* first pad of each type gets a selector + preroll queue */
-    if (group->type[type - 1].npads == 0) {
-      GST_DEBUG ("play base: pad needs new preroll");
-      gen_preroll_element (play_base_bin, group, type, pad, info);
-    }
+  gst_caps_unref (caps);
+
+  if (type == GST_STREAM_TYPE_UNKNOWN) {
+    /* Unknown streams get added to the group, but the data
+     * just gets ignored */
+    add_stream (group, info);
+
+    GROUP_UNLOCK (play_base_bin);
+
+    /* signal the no more pads after adding the stream */
+    if (last)
+      no_more_pads_full (element, is_subs, play_base_bin);
+
+    return;
+  }
+
+  /* first pad of each type gets a selector + preroll queue */
+  first_pad = (group->type[type - 1].npads == 0);
+
+  if (first_pad) {
+    GST_DEBUG ("play base: pad needs new preroll");
+    gen_preroll_element (play_base_bin, group, type, pad, info);
   }
 
   /* add to stream selector */
@@ -1248,12 +1331,8 @@ new_decoded_pad_full (GstElement * element, GstPad * pad, gboolean last,
   gst_pad_link (pad, sinkpad);
   gst_object_unref (sinkpad);
 
-  /* add the stream to the list */
-  gst_caps_unref (caps);
-  info->origin = GST_OBJECT_CAST (pad);
-
   /* select 1st for now - we'll select a preferred one after preroll */
-  if (type == GST_STREAM_TYPE_UNKNOWN || group->type[type - 1].npads > 0) {
+  if (!first_pad) {
     guint id;
 
     GST_DEBUG ("Adding silence_stream data probe on type %d (npads %d)", type,
@@ -1264,6 +1343,7 @@ new_decoded_pad_full (GstElement * element, GstPad * pad, gboolean last,
     g_object_set_data (G_OBJECT (pad), "eat_probe", GINT_TO_POINTER (id));
   }
 
+  /* add the stream to the list */
   add_stream (group, info);
 
   GROUP_UNLOCK (play_base_bin);
@@ -1386,8 +1466,17 @@ unknown_uri:
     gchar *prot = gst_uri_get_protocol (sub_uri);
 
     if (prot) {
-      GST_ELEMENT_ERROR (play_base_bin, RESOURCE, FAILED,
-          (_("No URI handler implemented for \"%s\"."), prot), (NULL));
+      gchar *desc;
+
+      gst_element_post_message (GST_ELEMENT (play_base_bin),
+          gst_missing_uri_source_message_new (GST_ELEMENT (play_base_bin),
+              prot));
+
+      desc = gst_pb_utils_get_source_description (prot);
+      GST_ELEMENT_ERROR (play_base_bin, CORE, MISSING_PLUGIN,
+          (_("A %s plugin is required to play this stream, but not installed."),
+              desc), ("No URI handler to handle sub_uri: %s", sub_uri));
+      g_free (desc);
       g_free (prot);
     } else
       goto invalid_uri;
@@ -1416,7 +1505,7 @@ static const gchar *stream_uris[] = { "http://", "mms://", "mmsh://",
 };
 
 /* blacklisted URIs, we know they will always fail. */
-static const gchar *blacklisted_uris[] = { "rtsp://", NULL };
+static const gchar *blacklisted_uris[] = { NULL };
 
 /* mime types that we don't consider to be media types */
 static const gchar *no_media_mimes[] = {
@@ -1503,8 +1592,17 @@ no_source:
     /* whoops, could not create the source element, dig a little deeper to
      * figure out what might be wrong. */
     if (prot) {
-      GST_ELEMENT_ERROR (play_base_bin, RESOURCE, FAILED,
-          (_("No URI handler implemented for \"%s\"."), prot), (NULL));
+      gchar *desc;
+
+      gst_element_post_message (GST_ELEMENT (play_base_bin),
+          gst_missing_uri_source_message_new (GST_ELEMENT (play_base_bin),
+              prot));
+
+      desc = gst_pb_utils_get_source_description (prot);
+      GST_ELEMENT_ERROR (play_base_bin, CORE, MISSING_PLUGIN,
+          (_("A %s plugin is required to play this stream, but not installed."),
+              desc), ("No URI handler for %s", prot));
+      g_free (desc);
       g_free (prot);
     } else
       goto invalid_uri;
@@ -1539,6 +1637,8 @@ source_new_pad (GstElement * element, GstPad * pad, GstPlayBaseBin * bin)
   if (!gst_element_link (bin->source, decoder))
     goto could_not_link;
 
+  gst_element_set_state (decoder, GST_STATE_PAUSED);
+
   return;
 
   /* ERRORS */
@@ -1691,7 +1791,7 @@ analyse_source (GstPlayBaseBin * play_base_bin, gboolean * is_raw,
 
   pads_iter = gst_element_iterate_src_pads (play_base_bin->source);
   while (!done) {
-    GstPad *pad;
+    GstPad *pad = NULL;
 
     switch (gst_iterator_next (pads_iter, (gpointer) & pad)) {
       case GST_ITERATOR_ERROR:
@@ -1712,13 +1812,18 @@ analyse_source (GstPlayBaseBin * play_base_bin, gboolean * is_raw,
         *have_out = TRUE;
 
         /* if FALSE, this pad has no caps and we continue with the next pad. */
-        if (!has_all_raw_caps (pad, is_raw))
+        if (!has_all_raw_caps (pad, is_raw)) {
+          gst_object_unref (pad);
           break;
+        }
 
         /* caps on source pad are all raw, we can add the pad */
-        if (*is_raw)
+        if (*is_raw) {
           new_decoded_pad_full (play_base_bin->source, pad, FALSE,
               play_base_bin, FALSE);
+        }
+
+        gst_object_unref (pad);
         break;
     }
   }
@@ -1796,6 +1901,9 @@ make_decoder (GstPlayBaseBin * play_base_bin)
   g_object_set_data (G_OBJECT (decoder), "pending", "1");
   play_base_bin->pending++;
 
+  GST_DEBUG_OBJECT (play_base_bin, "created decodebin, %d pending",
+      play_base_bin->pending);
+
   play_base_bin->decoders = g_slist_prepend (play_base_bin->decoders, decoder);
 
   return decoder;
@@ -1817,7 +1925,6 @@ remove_source (GstPlayBaseBin * bin)
   if (source) {
     GST_DEBUG_OBJECT (bin, "removing old src element");
     gst_element_set_state (source, GST_STATE_NULL);
-    gst_bin_remove (GST_BIN_CAST (bin), source);
 
     if (bin->src_np_sig_id) {
       g_signal_handler_disconnect (G_OBJECT (source), bin->src_np_sig_id);
@@ -1827,6 +1934,7 @@ remove_source (GstPlayBaseBin * bin)
       g_signal_handler_disconnect (G_OBJECT (source), bin->src_nmp_sig_id);
       bin->src_nmp_sig_id = 0;
     }
+    gst_bin_remove (GST_BIN_CAST (bin), source);
     bin->source = NULL;
   }
 }
@@ -1882,9 +1990,13 @@ setup_source (GstPlayBaseBin * play_base_bin)
   /* remove our previous preroll queues */
   remove_groups (play_base_bin);
 
+  /* clear pending dynamic elements */
+  play_base_bin->pending = 0;
+
   /* do subs */
   if (subbin) {
     GstElement *db;
+    GstBus *bus;
 
     play_base_bin->subtitle = subbin;
     db = gst_bin_get_by_name (GST_BIN_CAST (subbin), "subtitle-decoder");
@@ -1899,6 +2011,9 @@ setup_source (GstPlayBaseBin * play_base_bin)
     g_object_set_data (G_OBJECT (db), "pending", "1");
     play_base_bin->pending++;
 
+    GST_DEBUG_OBJECT (play_base_bin, "we have subtitles, %d pending",
+        play_base_bin->pending);
+
     if (!play_base_bin->is_stream) {
       GstStateChangeReturn sret;
 
@@ -1914,16 +2029,14 @@ setup_source (GstPlayBaseBin * play_base_bin)
        * to commit the subtitle group using an extra flag. */
       play_base_bin->subtitle_done = FALSE;
 
+      /* since subbin is still a stand-alone bin, we need to add a custom bus
+       * to intercept error messages, so we can stop waiting and continue */
+      bus = gst_bus_new ();
+      gst_element_set_bus (subbin, bus);
+      gst_bus_set_sync_handler (bus, subbin_startup_sync_msg, play_base_bin);
+
       sret = gst_element_set_state (subbin, GST_STATE_PAUSED);
       if (sret != GST_STATE_CHANGE_FAILURE) {
-        GstBus *bus;
-
-        /* since subbin is still a stand-alone bin, we need to add a custom bus
-         * to intercept error messages, so we can stop waiting and continue */
-        bus = gst_bus_new ();
-        gst_element_set_bus (subbin, bus);
-        gst_bus_set_sync_handler (bus, subbin_startup_sync_msg, play_base_bin);
-
         GROUP_LOCK (play_base_bin);
         GST_DEBUG ("waiting for subtitle to complete...");
         while (!play_base_bin->subtitle_done)
@@ -1931,10 +2044,6 @@ setup_source (GstPlayBaseBin * play_base_bin)
         GST_DEBUG ("group done !");
         GROUP_UNLOCK (play_base_bin);
 
-        gst_bus_set_sync_handler (bus, NULL, NULL);
-        gst_element_set_bus (subbin, NULL);
-        gst_object_unref (bus);
-
         if (!play_base_bin->building_group ||
             play_base_bin->building_group->type[GST_STREAM_TYPE_TEXT -
                 1].npads == 0) {
@@ -1952,7 +2061,12 @@ setup_source (GstPlayBaseBin * play_base_bin)
         gst_object_unref (play_base_bin->subtitle);
         play_base_bin->subtitle = NULL;
       }
+
+      gst_bus_set_sync_handler (bus, NULL, NULL);
+      gst_element_set_bus (subbin, NULL);
+      gst_object_unref (bus);
     }
+    gst_object_unref (db);
   }
   /* see if the source element emits raw audio/video all by itself,
    * if so, we can create streams for the pads and be done with it.
@@ -1977,7 +2091,6 @@ setup_source (GstPlayBaseBin * play_base_bin)
     return TRUE;
   }
   if (is_dynamic) {
-    GST_DEBUG_OBJECT (play_base_bin, "Source has dynamic output pads");
     /* connect a handler for the new-pad signal */
     play_base_bin->src_np_sig_id =
         g_signal_connect (G_OBJECT (play_base_bin->source), "pad-added",
@@ -1987,6 +2100,8 @@ setup_source (GstPlayBaseBin * play_base_bin)
         G_CALLBACK (source_no_more_pads), play_base_bin);
     g_object_set_data (G_OBJECT (play_base_bin->source), "pending", "1");
     play_base_bin->pending++;
+    GST_DEBUG_OBJECT (play_base_bin,
+        "Source has dynamic output pads, %d pending", play_base_bin->pending);
   } else {
     GstElement *decoder;
 
@@ -2419,8 +2534,10 @@ gst_play_base_bin_get_property (GObject * object, guint prop_id, GValue * value,
           (gpointer) gst_play_base_bin_get_streaminfo (play_base_bin));
       break;
     case ARG_STREAMINFO_VALUES:{
-      g_value_set_boxed (value,
-          gst_play_base_bin_get_streaminfo_value_array (play_base_bin));
+      GValueArray *copy;
+
+      copy = gst_play_base_bin_get_streaminfo_value_array (play_base_bin);
+      g_value_take_boxed (value, copy);
       break;
     }
     case ARG_SOURCE:
@@ -2481,7 +2598,9 @@ gst_play_base_bin_change_state (GstElement * element, GstStateChange transition)
 
       finish_source (play_base_bin);
       break;
+      /* clean-up in both cases, READY=>NULL clean-up is if there was an error */
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+    case GST_STATE_CHANGE_READY_TO_NULL:
       play_base_bin->need_rebuild = TRUE;
       remove_decoders (play_base_bin);
       remove_groups (play_base_bin);
@@ -2509,7 +2628,7 @@ cleanup_groups:
   }
 }
 
-const GList *
+static const GList *
 gst_play_base_bin_get_streaminfo (GstPlayBaseBin * play_base_bin)
 {
   GstPlayBaseGroup *group = get_active_group (play_base_bin);
@@ -2521,15 +2640,20 @@ gst_play_base_bin_get_streaminfo (GstPlayBaseBin * play_base_bin)
   return info;
 }
 
-const GValueArray *
+static GValueArray *
 gst_play_base_bin_get_streaminfo_value_array (GstPlayBaseBin * play_base_bin)
 {
-  GstPlayBaseGroup *group = get_active_group (play_base_bin);
+  GstPlayBaseGroup *group;
   GValueArray *array = NULL;
 
+  GROUP_LOCK (play_base_bin);
+  group = get_active_group (play_base_bin);
   if (group) {
-    array = group->streaminfo_value_array;
+    array = g_value_array_copy (group->streaminfo_value_array);
+  } else {
+    array = g_value_array_new (0);
   }
+  GROUP_UNLOCK (play_base_bin);
 
   return array;
 }
