@@ -1,9 +1,8 @@
 /* GStreamer
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
- *                    2000 Wim Taymans <wtay@chello.be>
  *                    2003 Colin Walters <cwalters@gnome.org>
- *                    2005 Wim Taymans <wim@fluendo.com>
- *                    2007 Wim Taymans <wim@fluendo.com>
+ *                    2000,2005,2007 Wim Taymans <wim@fluendo.com>
+ *                    2007 Thiago Sousa Santos <thiagoss@lcc.ufcg.edu.br>
  *
  * gstqueue2.c:
  *
@@ -42,11 +41,19 @@
  * The default queue size limits are 100 buffers, 2MB of data, or
  * two seconds worth of data, whichever is reached first.
  *
+ * If you set temp-location, the element will buffer data on the file
+ * specified by it. By using this, it will buffer the entire 
+ * stream data on the file independently of the queue size limits, they
+ * will only be used for buffering statistics.
+ *
  */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <glib/gstdio.h>
+
 #include <gst/gst.h>
 #include <gst/gst-i18n-plugin.h>
 
@@ -83,6 +90,10 @@ enum
 #define DEFAULT_LOW_PERCENT        10
 #define DEFAULT_HIGH_PERCENT       99
 
+/* other defines */
+#define DEFAULT_BUFFER_SIZE 4096
+#define QUEUE_IS_USING_TEMP_FILE(queue) (queue->temp_location != NULL)
+
 enum
 {
   PROP_0,
@@ -109,6 +120,8 @@ enum
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_QUEUE))
 #define GST_IS_QUEUE_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_QUEUE))
+#define GST_QUEUE_CAST(obj) \
+  ((GstQueue *)(obj))
 
 typedef struct _GstQueue GstQueue;
 typedef struct _GstQueueSize GstQueueSize;
@@ -176,6 +189,14 @@ struct _GstQueue
 
   /* temp location stuff */
   gchar *temp_location;
+  FILE *temp_file;
+  guint64 writing_pos;
+  guint64 reading_pos;
+  /* we need this to send the first new segment event of the stream
+   * because we can't save it on the file */
+  gboolean segment_event_received;
+  GstEvent *starting_segment;
+
 };
 
 struct _GstQueueClass
@@ -187,7 +208,7 @@ struct _GstQueueClass
   GST_CAT_LOG_OBJECT (queue_dataflow, queue, \
                       "(%s:%s) " msg ": %u of %u buffers, %u of %u " \
                       "bytes, %" G_GUINT64_FORMAT " of %" G_GUINT64_FORMAT \
-                      " ns, %u items", \
+                      " ns, %"G_GUINT64_FORMAT" items", \
                       GST_DEBUG_PAD_NAME (pad), \
                       queue->cur_level.buffers, \
                       queue->max_level.buffers, \
@@ -195,7 +216,9 @@ struct _GstQueueClass
                       queue->max_level.bytes, \
                       queue->cur_level.time, \
                       queue->max_level.time, \
-                      queue->queue->length)
+                      (guint64) (QUEUE_IS_USING_TEMP_FILE(queue) ? \
+                        queue->writing_pos - queue->reading_pos : \
+                        queue->queue->length))
 
 #define GST_QUEUE_MUTEX_LOCK(q) G_STMT_START {                          \
   g_mutex_lock (q->qlock);                                              \
@@ -304,6 +327,11 @@ static gboolean gst_queue_handle_src_query (GstPad * pad, GstQuery * query);
 
 static GstCaps *gst_queue_getcaps (GstPad * pad);
 
+static GstFlowReturn gst_queue_get_range (GstPad * pad, guint64 offset,
+    guint length, GstBuffer ** buffer);
+static gboolean gst_queue_src_checkgetrange_function (GstPad * pad);
+
+static gboolean gst_queue_src_activate_pull (GstPad * pad, gboolean active);
 static gboolean gst_queue_src_activate_push (GstPad * pad, gboolean active);
 static gboolean gst_queue_sink_activate_push (GstPad * pad, gboolean active);
 static GstStateChangeReturn gst_queue_change_state (GstElement * element,
@@ -371,7 +399,7 @@ gst_queue_class_init (GstQueueClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_TEMP_LOCATION,
       g_param_spec_string ("temp-location", "Temporary File Location",
-          "Location of a temporary file to store data in (unused)",
+          "Location of a temporary file to store data in",
           NULL, G_PARAM_READWRITE));
 
   gst_element_class_add_pad_template (gstelement_class,
@@ -406,8 +434,14 @@ gst_queue_init (GstQueue * queue, GstQueueClass * g_class)
 
   queue->srcpad = gst_pad_new_from_static_template (&srctemplate, "src");
 
+  gst_pad_set_activatepull_function (queue->srcpad,
+      GST_DEBUG_FUNCPTR (gst_queue_src_activate_pull));
   gst_pad_set_activatepush_function (queue->srcpad,
       GST_DEBUG_FUNCPTR (gst_queue_src_activate_push));
+  gst_pad_set_getrange_function (queue->srcpad,
+      GST_DEBUG_FUNCPTR (gst_queue_get_range));
+  gst_pad_set_checkgetrange_function (queue->srcpad,
+      GST_DEBUG_FUNCPTR (gst_queue_src_checkgetrange_function));
   gst_pad_set_acceptcaps_function (queue->srcpad,
       GST_DEBUG_FUNCPTR (gst_queue_acceptcaps));
   gst_pad_set_getcaps_function (queue->srcpad,
@@ -443,6 +477,10 @@ gst_queue_init (GstQueue * queue, GstQueueClass * g_class)
   queue->item_del = g_cond_new ();
   queue->queue = g_queue_new ();
 
+  /* tempfile related */
+  queue->temp_location = NULL;
+  queue->temp_file = NULL;
+
   GST_DEBUG_OBJECT (queue,
       "initialized queue's not_empty & not_full conditions");
 }
@@ -460,12 +498,17 @@ gst_queue_finalize (GObject * object)
 
     gst_mini_object_unref (data);
   }
+
   g_queue_free (queue->queue);
   g_mutex_free (queue->qlock);
   g_cond_free (queue->item_add);
   g_cond_free (queue->item_del);
   g_timer_destroy (queue->timer);
 
+  /* temp_file path cleanup  */
+  if (queue->temp_location != NULL)
+    g_free (queue->temp_location);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -545,6 +588,16 @@ apply_segment (GstQueue * queue, GstEvent * event, GstSegment * segment)
   gst_event_parse_new_segment_full (event, &update, &rate, &arate,
       &format, &start, &stop, &time);
 
+  GST_DEBUG_OBJECT (queue,
+      "received NEWSEGMENT update %d, rate %lf, applied rate %lf, "
+      "format %d, "
+      "%" G_GINT64_FORMAT " -- %" G_GINT64_FORMAT ", time %"
+      G_GINT64_FORMAT ", accum %" G_GINT64_FORMAT, update, rate, arate,
+      format, start, stop, time);
+
+  if (format == GST_FORMAT_BYTES) {
+  }
+
   /* now configure the values, we use these to track timestamps on the
    * sinkpad. */
   if (format != GST_FORMAT_TIME) {
@@ -599,7 +652,7 @@ update_buffering (GstQueue * queue)
   gint percent;
   gboolean post = FALSE;
 
-  if (!queue->use_buffering)
+  if (!queue->use_buffering || queue->high_percent <= 0)
     return;
 
 #define GET_PERCENT(format) ((queue->max_level.format) > 0 ? \
@@ -614,6 +667,8 @@ update_buffering (GstQueue * queue)
     percent = GET_PERCENT (bytes);
     percent = MAX (percent, GET_PERCENT (time));
     percent = MAX (percent, GET_PERCENT (buffers));
+
+    /* also apply the rate estimate when we need to */
     if (queue->use_rate_estimate)
       percent = MAX (percent, GET_PERCENT (rate_time));
   }
@@ -659,11 +714,12 @@ reset_rate_timer (GstQueue * queue)
   queue->timer_started = FALSE;
 }
 
+/* the interval in seconds to recalculate the rate */
+#define RATE_INTERVAL    0.2
 /* Tuning for rate estimation. We use a large window for the input rate because
  * it should be stable when connected to a network. The output rate is less
  * stable (the elements preroll, queues behind a demuxer fill, ...) and should
  * therefore adapt more quickly. */
-#define RATE_INTERVAL    0.5
 #define AVG_IN(avg,val)  ((avg) * 15.0 + (val)) / 16.0
 #define AVG_OUT(avg,val) ((avg) * 3.0 + (val)) / 4.0
 
@@ -719,20 +775,222 @@ update_rates (GstQueue * queue)
       GST_TIME_ARGS (queue->cur_level.rate_time));
 }
 
+static void
+gst_queue_write_buffer_to_file (GstQueue * queue, GstBuffer * buffer)
+{
+  guint size;
+  guint8 *data;
+  int ret;
+
+  fseek (queue->temp_file, queue->writing_pos, SEEK_SET);
+
+  data = GST_BUFFER_DATA (buffer);
+  size = GST_BUFFER_SIZE (buffer);
+
+  ret = fwrite (data, 1, size, queue->temp_file);
+  if (ret < size) {
+    /* FIXME do something useful here */
+    GST_ERROR_OBJECT (queue, "fwrite returned error");
+  }
+  queue->writing_pos += size;
+}
+
+/* see if there is enough data in the file to read a full buffer */
+static gboolean
+gst_queue_have_data (GstQueue * queue, guint64 offset, guint length)
+{
+  GST_DEBUG_OBJECT (queue,
+      "offset %" G_GUINT64_FORMAT ", len %u, write %" G_GUINT64_FORMAT, offset,
+      length, queue->writing_pos);
+  if (queue->is_eos)
+    return TRUE;
+
+  if (offset + length < queue->writing_pos)
+    return TRUE;
+
+  return FALSE;
+}
+
+static GstFlowReturn
+gst_queue_create_read (GstQueue * queue, guint64 offset, guint length,
+    GstBuffer ** buffer)
+{
+  size_t res;
+  GstBuffer *buf;
+
+  /* check if we have enough data at @offset. If there is not enough data, we
+   * block and wait. */
+  while (!gst_queue_have_data (queue, offset, length)) {
+    GST_QUEUE_WAIT_ADD_CHECK (queue, out_flushing);
+  }
+
+#ifdef HAVE_FSEEKO
+  if (fseeko (queue->temp_file, (off_t) offset, SEEK_SET) != 0)
+    goto seek_failed;
+#elif defined (G_OS_UNIX)
+  if (lseek (fileno (queue->temp_file), (off_t) offset,
+          SEEK_SET) == (off_t) - 1)
+    goto seek_failed;
+#else
+  if (fseek (queue->temp_file, (long) offset, SEEK_SET) != 0)
+    goto seek_failed;
+#endif
+
+  buf = gst_buffer_new_and_alloc (length);
+
+  /* this should not block */
+  GST_LOG_OBJECT (queue, "Reading %d bytes", length);
+  res = fread (GST_BUFFER_DATA (buf), 1, length, queue->temp_file);
+  GST_LOG_OBJECT (queue, "read %" G_GSIZE_FORMAT " bytes", res);
+
+  if (G_UNLIKELY (res == 0)) {
+    /* check for errors or EOF */
+    if (ferror (queue->temp_file))
+      goto could_not_read;
+    if (feof (queue->temp_file) && length > 0)
+      goto eos;
+  }
+
+  length = res;
+
+  GST_BUFFER_SIZE (buf) = length;
+  GST_BUFFER_OFFSET (buf) = offset;
+  GST_BUFFER_OFFSET_END (buf) = offset + length;
+
+  *buffer = buf;
+
+  queue->reading_pos = offset + length;
+
+  return GST_FLOW_OK;
+
+  /* ERRORS */
+out_flushing:
+  {
+    GST_DEBUG_OBJECT (queue, "we are flushing");
+    return GST_FLOW_WRONG_STATE;
+  }
+seek_failed:
+  {
+    GST_ELEMENT_ERROR (queue, RESOURCE, READ, (NULL), GST_ERROR_SYSTEM);
+    return GST_FLOW_ERROR;
+  }
+could_not_read:
+  {
+    GST_ELEMENT_ERROR (queue, RESOURCE, READ, (NULL), GST_ERROR_SYSTEM);
+    gst_buffer_unref (buf);
+    return GST_FLOW_ERROR;
+  }
+eos:
+  {
+    GST_DEBUG ("non-regular file hits EOS");
+    gst_buffer_unref (buf);
+    return GST_FLOW_UNEXPECTED;
+  }
+}
+
+/* should be called with QUEUE_LOCK */
+static GstMiniObject *
+gst_queue_read_item_from_file (GstQueue * queue)
+{
+  GstMiniObject *item;
+
+  if (queue->starting_segment != NULL) {
+    item = GST_MINI_OBJECT_CAST (queue->starting_segment);
+    queue->starting_segment = NULL;
+  } else {
+    GstFlowReturn ret;
+    GstBuffer *buffer;
+
+    ret =
+        gst_queue_create_read (queue, queue->reading_pos, DEFAULT_BUFFER_SIZE,
+        &buffer);
+    switch (ret) {
+      case GST_FLOW_OK:
+        item = GST_MINI_OBJECT_CAST (buffer);
+        break;
+      case GST_FLOW_UNEXPECTED:
+        item = GST_MINI_OBJECT_CAST (gst_event_new_eos ());
+        break;
+      default:
+        item = NULL;
+        break;
+    }
+  }
+  return item;
+}
+
+static gboolean
+gst_queue_open_temp_location_file (GstQueue * queue)
+{
+  /* nothing to do */
+  if (queue->temp_location == NULL)
+    goto no_filename;
+
+  /* open the file for update/writing */
+  queue->temp_file = g_fopen (queue->temp_location, "wb+");
+  /* error creating file */
+  if (queue->temp_file == NULL)
+    goto open_failed;
+
+  queue->writing_pos = 0;
+  queue->reading_pos = 0;
+
+  return TRUE;
+
+  /* ERRORS */
+no_filename:
+  {
+    GST_ELEMENT_ERROR (queue, RESOURCE, NOT_FOUND,
+        (_("No file name specified.")), (NULL));
+    return FALSE;
+  }
+open_failed:
+  {
+    GST_ELEMENT_ERROR (queue, RESOURCE, OPEN_READ,
+        (_("Could not open file \"%s\" for reading."), queue->temp_location),
+        GST_ERROR_SYSTEM);
+    return FALSE;
+  }
+}
+
+static void
+gst_queue_close_temp_location_file (GstQueue * queue)
+{
+  /* nothing to do */
+  if (queue->temp_file == NULL)
+    return;
+
+  /* we don't remove the file so that the application can use it as a cache
+   * later on */
+  fflush (queue->temp_file);
+  fclose (queue->temp_file);
+  remove (queue->temp_location);
+  queue->temp_file = NULL;
+}
+
 static void
 gst_queue_locked_flush (GstQueue * queue)
 {
-  while (!g_queue_is_empty (queue->queue)) {
-    GstMiniObject *data = g_queue_pop_head (queue->queue);
+  if (QUEUE_IS_USING_TEMP_FILE (queue)) {
+    gst_queue_close_temp_location_file (queue);
+    gst_queue_open_temp_location_file (queue);
+  } else {
+    while (!g_queue_is_empty (queue->queue)) {
+      GstMiniObject *data = g_queue_pop_head (queue->queue);
 
-    /* Then lose another reference because we are supposed to destroy that
-       data when flushing */
-    gst_mini_object_unref (data);
+      /* Then lose another reference because we are supposed to destroy that
+         data when flushing */
+      gst_mini_object_unref (data);
+    }
   }
   GST_QUEUE_CLEAR_LEVEL (queue->cur_level);
   gst_segment_init (&queue->sink_segment, GST_FORMAT_TIME);
   gst_segment_init (&queue->src_segment, GST_FORMAT_TIME);
   queue->is_eos = FALSE;
+  if (queue->starting_segment != NULL)
+    gst_event_unref (queue->starting_segment);
+  queue->starting_segment = NULL;
+  queue->segment_event_received = FALSE;
 
   /* we deleted a lot of something */
   GST_QUEUE_SIGNAL_DEL (queue);
@@ -757,8 +1015,10 @@ gst_queue_locked_enqueue (GstQueue * queue, gpointer item)
     apply_buffer (queue, buffer, &queue->sink_segment);
     /* update the byterate stats */
     update_rates (queue);
-    /* update the buffering status */
-    update_buffering (queue);
+
+    if (QUEUE_IS_USING_TEMP_FILE (queue)) {
+      gst_queue_write_buffer_to_file (queue, buffer);
+    }
 
   } else if (GST_IS_EVENT (item)) {
     GstEvent *event;
@@ -773,8 +1033,19 @@ gst_queue_locked_enqueue (GstQueue * queue, gpointer item)
         break;
       case GST_EVENT_NEWSEGMENT:
         apply_segment (queue, event, &queue->sink_segment);
+        /* This is our first new segment, we hold it
+         * as we can't save it on the temp file */
+        if (QUEUE_IS_USING_TEMP_FILE (queue)) {
+          if (queue->segment_event_received)
+            goto unexpected_event;
+
+          queue->segment_event_received = TRUE;
+          queue->starting_segment = event;
+        }
         break;
       default:
+        if (QUEUE_IS_USING_TEMP_FILE (queue))
+          goto unexpected_event;
         break;
     }
   } else {
@@ -784,9 +1055,27 @@ gst_queue_locked_enqueue (GstQueue * queue, gpointer item)
     item = NULL;
   }
 
-  if (item)
-    g_queue_push_tail (queue->queue, item);
-  GST_QUEUE_SIGNAL_ADD (queue);
+  if (item) {
+    /* update the buffering status */
+    update_buffering (queue);
+
+    if (!QUEUE_IS_USING_TEMP_FILE (queue))
+      g_queue_push_tail (queue->queue, item);
+    GST_QUEUE_SIGNAL_ADD (queue);
+  }
+
+  return;
+
+  /* ERRORS */
+unexpected_event:
+  {
+    g_warning
+        ("Unexpected event of kind %s can't be added in temp file of queue %s ",
+        gst_event_type_get_name (GST_EVENT_TYPE (item)),
+        GST_OBJECT_NAME (queue));
+    gst_event_unref (GST_EVENT_CAST (item));
+    return;
+  }
 }
 
 /* dequeue an item from the queue and update level stats */
@@ -795,7 +1084,11 @@ gst_queue_locked_dequeue (GstQueue * queue)
 {
   GstMiniObject *item;
 
-  item = g_queue_pop_head (queue->queue);
+  if (QUEUE_IS_USING_TEMP_FILE (queue))
+    item = gst_queue_read_item_from_file (queue);
+  else
+    item = g_queue_pop_head (queue->queue);
+
   if (item == NULL)
     goto no_item;
 
@@ -915,6 +1208,9 @@ done:
   /* ERRORS */
 out_flushing:
   {
+    GST_DEBUG_OBJECT (queue, "we are flushing");
+    GST_QUEUE_MUTEX_UNLOCK (queue);
+
     gst_buffer_unref (event);
     return FALSE;
   }
@@ -927,8 +1223,12 @@ gst_queue_is_empty (GstQueue * queue)
   if (queue->is_eos)
     return FALSE;
 
-  if (queue->queue->length == 0)
-    return TRUE;
+  if (QUEUE_IS_USING_TEMP_FILE (queue)) {
+    return queue->writing_pos == queue->reading_pos;
+  } else {
+    if (queue->queue->length == 0)
+      return TRUE;
+  }
 
   return FALSE;
 }
@@ -942,6 +1242,10 @@ gst_queue_is_filled (GstQueue * queue)
   if (queue->is_eos)
     return TRUE;
 
+  /* if using file, we're never filled if we don't have EOS */
+  if (QUEUE_IS_USING_TEMP_FILE (queue))
+    return FALSE;
+
 #define CHECK_FILLED(format) ((queue->max_level.format) > 0 && \
 		(queue->cur_level.format) >= (queue->max_level.format))
 
@@ -996,6 +1300,7 @@ out_flushing:
     GST_CAT_LOG_OBJECT (queue_dataflow, queue,
         "exit because task paused, reason: %s", gst_flow_get_name (ret));
     GST_QUEUE_MUTEX_UNLOCK (queue);
+
     gst_buffer_unref (buffer);
 
     return ret;
@@ -1052,6 +1357,8 @@ out_flushing:
   }
 }
 
+/* called repeadedly with @pad as the source pad. This function should push out
+ * data to the peer element. */
 static void
 gst_queue_loop (GstPad * pad)
 {
@@ -1104,20 +1411,25 @@ gst_queue_handle_src_event (GstPad * pad, GstEvent * event)
   return res;
 }
 
+static gboolean
+gst_queue_peer_query (GstQueue * queue, GstPad * pad, GstQuery * query)
+{
+  gboolean ret = FALSE;
+  GstPad *peer;
+
+  if ((peer = gst_pad_get_peer (pad))) {
+    ret = gst_pad_query (peer, query);
+    gst_object_unref (peer);
+  }
+  return ret;
+}
+
 static gboolean
 gst_queue_handle_src_query (GstPad * pad, GstQuery * query)
 {
-  GstQueue *queue = GST_QUEUE (GST_PAD_PARENT (pad));
-  GstPad *peer;
-  gboolean res;
+  GstQueue *queue;
 
-  if (!(peer = gst_pad_get_peer (queue->sinkpad)))
-    return FALSE;
-
-  res = gst_pad_query (peer, query);
-  gst_object_unref (peer);
-  if (!res)
-    return FALSE;
+  queue = GST_QUEUE (GST_PAD_PARENT (pad));
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_POSITION:
@@ -1125,6 +1437,9 @@ gst_queue_handle_src_query (GstPad * pad, GstQuery * query)
       gint64 peer_pos;
       GstFormat format;
 
+      if (!gst_queue_peer_query (queue, queue->sinkpad, query))
+        goto peer_failed;
+
       /* get peer position */
       gst_query_parse_position (query, &format, &peer_pos);
 
@@ -1145,14 +1460,92 @@ gst_queue_handle_src_query (GstPad * pad, GstQuery * query)
       gst_query_set_position (query, format, peer_pos);
       break;
     }
+    case GST_QUERY_DURATION:
+    {
+      GST_DEBUG_OBJECT (queue, "waiting for preroll in duration query");
+
+      GST_QUEUE_MUTEX_LOCK (queue);
+      /* we have to wait until the upstream element is at least paused, which
+       * happened when we received a first item. */
+      while (gst_queue_is_empty (queue)) {
+        GST_QUEUE_WAIT_ADD_CHECK (queue, flushing);
+      }
+      GST_QUEUE_MUTEX_UNLOCK (queue);
+
+      if (!gst_queue_peer_query (queue, queue->sinkpad, query))
+        goto peer_failed;
+
+      GST_DEBUG_OBJECT (queue, "peer query success");
+      break;
+    }
     default:
       /* peer handled other queries */
+      if (!gst_queue_peer_query (queue, queue->sinkpad, query))
+        goto peer_failed;
       break;
   }
 
   return TRUE;
+
+  /* ERRORS */
+peer_failed:
+  {
+    GST_DEBUG_OBJECT (queue, "failed peer query");
+    return FALSE;
+  }
+flushing:
+  {
+    GST_DEBUG_OBJECT (queue, "flushing while waiting for query");
+    GST_QUEUE_MUTEX_UNLOCK (queue);
+    return FALSE;
+  }
 }
 
+static GstFlowReturn
+gst_queue_get_range (GstPad * pad, guint64 offset, guint length,
+    GstBuffer ** buffer)
+{
+  GstQueue *queue;
+  GstFlowReturn ret;
+
+  queue = GST_QUEUE_CAST (gst_pad_get_parent (pad));
+
+  GST_QUEUE_MUTEX_LOCK_CHECK (queue, out_flushing);
+  length = (length == -1) ? DEFAULT_BUFFER_SIZE : length;
+  offset = (offset == -1) ? queue->reading_pos : offset;
+
+  /* function will block when the range is not yet available */
+  ret = gst_queue_create_read (queue, offset, length, buffer);
+  GST_QUEUE_MUTEX_UNLOCK (queue);
+
+  gst_object_unref (queue);
+
+  return ret;
+
+  /* ERRORS */
+out_flushing:
+  {
+    GST_DEBUG_OBJECT (queue, "we are flushing");
+    GST_QUEUE_MUTEX_UNLOCK (queue);
+    return GST_FLOW_WRONG_STATE;
+  }
+}
+
+static gboolean
+gst_queue_src_checkgetrange_function (GstPad * pad)
+{
+  GstQueue *queue;
+  gboolean ret;
+
+  queue = GST_QUEUE (gst_pad_get_parent (pad));
+  /* we can operate in pull mode when we are using a tempfile */
+  ret = QUEUE_IS_USING_TEMP_FILE (queue);
+  gst_object_unref (GST_OBJECT (queue));
+
+  return ret;
+}
+
+/* sink currently only operates in push mode */
 static gboolean
 gst_queue_sink_activate_push (GstPad * pad, gboolean active)
 {
@@ -1163,12 +1556,14 @@ gst_queue_sink_activate_push (GstPad * pad, gboolean active)
 
   if (active) {
     GST_QUEUE_MUTEX_LOCK (queue);
+    GST_DEBUG_OBJECT (queue, "activating push mode");
     queue->srcresult = GST_FLOW_OK;
     reset_rate_timer (queue);
     GST_QUEUE_MUTEX_UNLOCK (queue);
   } else {
     /* unblock chain function */
     GST_QUEUE_MUTEX_LOCK (queue);
+    GST_DEBUG_OBJECT (queue, "deactivating push mode");
     queue->srcresult = GST_FLOW_WRONG_STATE;
     gst_queue_locked_flush (queue);
     GST_QUEUE_MUTEX_UNLOCK (queue);
@@ -1179,6 +1574,8 @@ gst_queue_sink_activate_push (GstPad * pad, gboolean active)
   return result;
 }
 
+/* src operating in push mode, we start a task on the source pad that pushes out
+ * buffers from the queue */
 static gboolean
 gst_queue_src_activate_push (GstPad * pad, gboolean active)
 {
@@ -1189,12 +1586,14 @@ gst_queue_src_activate_push (GstPad * pad, gboolean active)
 
   if (active) {
     GST_QUEUE_MUTEX_LOCK (queue);
+    GST_DEBUG_OBJECT (queue, "activating push mode");
     queue->srcresult = GST_FLOW_OK;
     result = gst_pad_start_task (pad, (GstTaskFunction) gst_queue_loop, pad);
     GST_QUEUE_MUTEX_UNLOCK (queue);
   } else {
     /* unblock loop function */
     GST_QUEUE_MUTEX_LOCK (queue);
+    GST_DEBUG_OBJECT (queue, "deactivating push mode");
     queue->srcresult = GST_FLOW_WRONG_STATE;
     /* the item add signal will unblock */
     g_cond_signal (queue->item_add);
@@ -1209,6 +1608,45 @@ gst_queue_src_activate_push (GstPad * pad, gboolean active)
   return result;
 }
 
+/* pull mode, downstream will call our getrange function */
+static gboolean
+gst_queue_src_activate_pull (GstPad * pad, gboolean active)
+{
+  gboolean result;
+  GstQueue *queue;
+
+  queue = GST_QUEUE (gst_pad_get_parent (pad));
+
+  if (active) {
+    if (QUEUE_IS_USING_TEMP_FILE (queue)) {
+      GST_QUEUE_MUTEX_LOCK (queue);
+      GST_DEBUG_OBJECT (queue, "activating pull mode");
+      queue->srcresult = GST_FLOW_OK;
+      result = TRUE;
+      GST_QUEUE_MUTEX_UNLOCK (queue);
+    } else {
+      GST_QUEUE_MUTEX_LOCK (queue);
+      GST_DEBUG_OBJECT (queue, "no temp file, cannot activate pull mode");
+      /* this is not allowed, we cannot operate in pull mode without a temp
+       * file. */
+      queue->srcresult = GST_FLOW_WRONG_STATE;
+      result = FALSE;
+      GST_QUEUE_MUTEX_UNLOCK (queue);
+    }
+  } else {
+    GST_QUEUE_MUTEX_LOCK (queue);
+    GST_DEBUG_OBJECT (queue, "deactivating pull mode");
+    queue->srcresult = GST_FLOW_WRONG_STATE;
+    /* this will unlock getrange */
+    g_cond_signal (queue->item_add);
+    result = TRUE;
+    GST_QUEUE_MUTEX_UNLOCK (queue);
+  }
+  gst_object_unref (queue);
+
+  return result;
+}
+
 static GstStateChangeReturn
 gst_queue_change_state (GstElement * element, GstStateChange transition)
 {
@@ -1221,6 +1659,12 @@ gst_queue_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
+      if (QUEUE_IS_USING_TEMP_FILE (queue)) {
+        if (!gst_queue_open_temp_location_file (queue))
+          ret = GST_STATE_CHANGE_FAILURE;
+      }
+      queue->segment_event_received = FALSE;
+      queue->starting_segment = NULL;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -1234,6 +1678,12 @@ gst_queue_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      if (QUEUE_IS_USING_TEMP_FILE (queue))
+        gst_queue_close_temp_location_file (queue);
+      if (queue->starting_segment != NULL) {
+        gst_event_unref (queue->starting_segment);
+        queue->starting_segment = NULL;
+      }
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
@@ -1257,6 +1707,35 @@ gst_queue_change_state (GstElement * element, GstStateChange transition)
 #define QUEUE_THRESHOLD_CHANGE(q)\
   g_cond_signal (queue->item_add);
 
+static gboolean
+gst_queue_set_temp_location (GstQueue * queue, const gchar * location)
+{
+  GstState state;
+
+  /* the element must be stopped in order to do this */
+  GST_OBJECT_LOCK (queue);
+  state = GST_STATE (queue);
+  if (state != GST_STATE_READY && state != GST_STATE_NULL)
+    goto wrong_state;
+  GST_OBJECT_UNLOCK (queue);
+
+  /* set new location */
+  g_free (queue->temp_location);
+  queue->temp_location = g_strdup (location);
+
+  g_object_notify (G_OBJECT (queue), "temp-location");
+
+  return TRUE;
+
+/* ERROR */
+wrong_state:
+  {
+    GST_DEBUG_OBJECT (queue, "setting temp-location in wrong state");
+    GST_OBJECT_UNLOCK (queue);
+    return FALSE;
+  }
+}
+
 static void
 gst_queue_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec)
@@ -1296,7 +1775,7 @@ gst_queue_set_property (GObject * object,
       queue->high_percent = g_value_get_int (value);
       break;
     case PROP_TEMP_LOCATION:
-      queue->temp_location = g_value_dup_string (value);
+      gst_queue_set_temp_location (queue, g_value_dup_string (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1359,8 +1838,8 @@ gst_queue_get_property (GObject * object,
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  GST_DEBUG_CATEGORY_INIT (queue_debug, "queue", 0, "queue element");
-  GST_DEBUG_CATEGORY_INIT (queue_dataflow, "queue_dataflow", 0,
+  GST_DEBUG_CATEGORY_INIT (queue_debug, "queue2", 0, "queue element");
+  GST_DEBUG_CATEGORY_INIT (queue_dataflow, "queue2_dataflow", 0,
       "dataflow inside the queue element");
 
 #ifdef ENABLE_NLS
