@@ -130,13 +130,10 @@ static GstStaticPadTemplate gst_video_rate_sink_template =
     GST_STATIC_CAPS ("video/x-raw-yuv; video/x-raw-rgb")
     );
 
-static void gst_video_rate_base_init (gpointer g_class);
-static void gst_video_rate_class_init (GstVideoRateClass * klass);
-static void gst_video_rate_init (GstVideoRate * videorate);
-
 static void gst_video_rate_swap_prev (GstVideoRate * videorate,
     GstBuffer * buffer, gint64 time);
 static gboolean gst_video_rate_event (GstPad * pad, GstEvent * event);
+static gboolean gst_video_rate_query (GstPad * pad, GstQuery * query);
 static GstFlowReturn gst_video_rate_chain (GstPad * pad, GstBuffer * buffer);
 
 static void gst_video_rate_set_property (GObject * object,
@@ -147,34 +144,9 @@ static void gst_video_rate_get_property (GObject * object,
 static GstStateChangeReturn gst_video_rate_change_state (GstElement * element,
     GstStateChange transition);
 
-static GstElementClass *parent_class = NULL;
-
 /*static guint gst_video_rate_signals[LAST_SIGNAL] = { 0 }; */
 
-static GType
-gst_video_rate_get_type (void)
-{
-  static GType video_rate_type = 0;
-
-  if (!video_rate_type) {
-    static const GTypeInfo video_rate_info = {
-      sizeof (GstVideoRateClass),
-      gst_video_rate_base_init,
-      NULL,
-      (GClassInitFunc) gst_video_rate_class_init,
-      NULL,
-      NULL,
-      sizeof (GstVideoRate),
-      0,
-      (GInstanceInitFunc) gst_video_rate_init,
-    };
-
-    video_rate_type = g_type_register_static (GST_TYPE_ELEMENT,
-        "GstVideoRate", &video_rate_info, 0);
-  }
-
-  return video_rate_type;
-}
+GST_BOILERPLATE (GstVideoRate, gst_video_rate, GstElement, GST_TYPE_ELEMENT);
 
 static void
 gst_video_rate_base_init (gpointer g_class)
@@ -288,6 +260,8 @@ gst_video_rate_setcaps (GstPad * pad, GstCaps * caps)
 
   videorate = GST_VIDEO_RATE (gst_pad_get_parent (pad));
 
+  GST_DEBUG ("setcaps called %" GST_PTR_FORMAT, caps);
+
   structure = gst_caps_get_structure (caps, 0);
   if (!gst_structure_get_fraction (structure, "framerate",
           &rate_numerator, &rate_denominator))
@@ -360,6 +334,11 @@ gst_video_rate_setcaps (GstPad * pad, GstCaps * caps)
     gst_object_unref (opeer);
   }
 done:
+  /* After a setcaps, our caps may have changed. In that case, we can't use
+   * the old buffer, if there was one (it might have different dimensions) */
+  GST_DEBUG ("swapping old buffers");
+  gst_video_rate_swap_prev (videorate, NULL, GST_CLOCK_TIME_NONE);
+
   gst_object_unref (videorate);
   return ret;
 
@@ -385,29 +364,30 @@ gst_video_rate_reset (GstVideoRate * videorate)
   videorate->out = 0;
   videorate->drop = 0;
   videorate->dup = 0;
-  videorate->next_ts = G_GINT64_CONSTANT (0);
+  videorate->next_ts = GST_CLOCK_TIME_NONE;
   gst_video_rate_swap_prev (videorate, NULL, 0);
 
   gst_segment_init (&videorate->segment, GST_FORMAT_TIME);
 }
 
 static void
-gst_video_rate_init (GstVideoRate * videorate)
+gst_video_rate_init (GstVideoRate * videorate, GstVideoRateClass * klass)
 {
   GST_DEBUG ("gst_video_rate_init");
   videorate->sinkpad =
       gst_pad_new_from_static_template (&gst_video_rate_sink_template, "sink");
-  gst_element_add_pad (GST_ELEMENT (videorate), videorate->sinkpad);
   gst_pad_set_event_function (videorate->sinkpad, gst_video_rate_event);
   gst_pad_set_chain_function (videorate->sinkpad, gst_video_rate_chain);
   gst_pad_set_getcaps_function (videorate->sinkpad, gst_video_rate_getcaps);
   gst_pad_set_setcaps_function (videorate->sinkpad, gst_video_rate_setcaps);
+  gst_element_add_pad (GST_ELEMENT (videorate), videorate->sinkpad);
 
   videorate->srcpad =
       gst_pad_new_from_static_template (&gst_video_rate_src_template, "src");
-  gst_element_add_pad (GST_ELEMENT (videorate), videorate->srcpad);
+  gst_pad_set_query_function (videorate->srcpad, gst_video_rate_query);
   gst_pad_set_getcaps_function (videorate->srcpad, gst_video_rate_getcaps);
   gst_pad_set_setcaps_function (videorate->srcpad, gst_video_rate_setcaps);
+  gst_element_add_pad (GST_ELEMENT (videorate), videorate->srcpad);
 
   gst_video_rate_reset (videorate);
   videorate->silent = DEFAULT_SILENT;
@@ -434,6 +414,9 @@ gst_video_rate_flush_prev (GstVideoRate * videorate)
   outbuf = gst_buffer_make_metadata_writable
       (gst_buffer_ref (videorate->prevbuf));
 
+  GST_BUFFER_OFFSET (outbuf) = videorate->out;
+  GST_BUFFER_OFFSET_END (outbuf) = videorate->out + 1;
+
   /* this is the timestamp we put on the buffer */
   push_ts = videorate->next_ts;
 
@@ -543,6 +526,62 @@ format_error:
   }
 }
 
+static gboolean
+gst_video_rate_query (GstPad * pad, GstQuery * query)
+{
+  GstVideoRate *videorate;
+  gboolean res = FALSE;
+
+  videorate = GST_VIDEO_RATE (gst_pad_get_parent (pad));
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_LATENCY:
+    {
+      GstClockTime min, max;
+      gboolean live;
+      guint64 latency;
+      GstPad *peer;
+
+      if ((peer = gst_pad_get_peer (videorate->sinkpad))) {
+        if ((res = gst_pad_query (peer, query))) {
+          gst_query_parse_latency (query, &live, &min, &max);
+
+          GST_DEBUG_OBJECT (videorate, "Peer latency: min %"
+              GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (min), GST_TIME_ARGS (max));
+
+          /* add latency. We don't really know since we hold on to the frames
+           * until we get a next frame, which can be anything. We assume
+           * however that this will take from_rate time. */
+          latency = gst_util_uint64_scale (GST_SECOND,
+              videorate->from_rate_denominator, videorate->from_rate_numerator);
+
+          GST_DEBUG_OBJECT (videorate, "Our latency: %"
+              GST_TIME_FORMAT, latency);
+
+          min += latency;
+          if (max != -1)
+            max += latency;
+
+          GST_DEBUG_OBJECT (videorate, "Calculated total latency : min %"
+              GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (min), GST_TIME_ARGS (max));
+
+          gst_query_set_latency (query, live, min, max);
+        }
+        gst_object_unref (peer);
+      }
+      break;
+    }
+    default:
+      res = gst_pad_query_default (pad, query);
+      break;
+  }
+  gst_object_unref (videorate);
+
+  return res;
+}
+
 static GstFlowReturn
 gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -550,7 +589,7 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
   GstFlowReturn res = GST_FLOW_OK;
   GstClockTime intime, in_ts;
 
-  videorate = GST_VIDEO_RATE (gst_pad_get_parent (pad));
+  videorate = GST_VIDEO_RATE (GST_PAD_PARENT (pad));
 
   /* make sure the denominators are not 0 */
   if (videorate->from_rate_denominator == 0 ||
@@ -559,6 +598,9 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
 
   in_ts = GST_BUFFER_TIMESTAMP (buffer);
 
+  if (G_UNLIKELY (in_ts == GST_CLOCK_TIME_NONE))
+    goto invalid_buffer;
+
   GST_DEBUG_OBJECT (videorate, "got buffer with timestamp %" GST_TIME_FORMAT,
       GST_TIME_ARGS (in_ts));
 
@@ -570,9 +612,11 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
   if (videorate->prevbuf == NULL) {
     gst_video_rate_swap_prev (videorate, buffer, intime);
     videorate->in++;
-    /* new buffer, we expect to output a buffer that matches the first
-     * timestamp in the segment */
-    videorate->next_ts = videorate->segment.start;
+    if (!GST_CLOCK_TIME_IS_VALID (videorate->next_ts)) {
+      /* new buffer, we expect to output a buffer that matches the first
+       * timestamp in the segment */
+      videorate->next_ts = videorate->segment.start;
+    }
   } else {
     GstClockTime prevtime;
     gint count = 0;
@@ -603,6 +647,7 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
 
     /* got 2 buffers, see which one is the best */
     do {
+
       diff1 = prevtime - videorate->next_ts;
       diff2 = intime - videorate->next_ts;
 
@@ -623,8 +668,10 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
         count++;
 
         /* on error the _flush function posted a warning already */
-        if ((res = gst_video_rate_flush_prev (videorate)) != GST_FLOW_OK)
+        if ((res = gst_video_rate_flush_prev (videorate)) != GST_FLOW_OK) {
+          gst_buffer_unref (buffer);
           goto done;
+        }
       }
       /* continue while the first one was the best */
     }
@@ -658,7 +705,6 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
     gst_video_rate_swap_prev (videorate, buffer, intime);
   }
 done:
-  gst_object_unref (videorate);
   return res;
 
   /* ERRORS */
@@ -669,6 +715,14 @@ not_negotiated:
     res = GST_FLOW_NOT_NEGOTIATED;
     goto done;
   }
+
+invalid_buffer:
+  {
+    GST_WARNING_OBJECT (videorate,
+        "Got buffer with GST_CLOCK_TIME_NONE timestamp, discarding it");
+    gst_buffer_unref (buffer);
+    goto done;
+  }
 }
 
 static void
