@@ -136,6 +136,7 @@ MotifWmHints, MwmHints;
 
 #define MWM_HINTS_DECORATIONS   (1L << 1)
 
+static void gst_ximagesink_reset (GstXImageSink * ximagesink);
 static void gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink,
     GstXImageBuffer * ximage);
 static void gst_ximagesink_xwindow_update_geometry (GstXImageSink * ximagesink,
@@ -164,8 +165,9 @@ enum
   PROP_DISPLAY,
   PROP_SYNCHRONOUS,
   PROP_PIXEL_ASPECT_RATIO,
-  PROP_FORCE_ASPECT_RATIO
-      /* FILL ME */
+  PROP_FORCE_ASPECT_RATIO,
+  PROP_HANDLE_EVENTS,
+  PROP_HANDLE_EXPOSE
 };
 
 static GstVideoSinkClass *parent_class = NULL;
@@ -201,7 +203,7 @@ gst_ximage_buffer_finalize (GstXImageBuffer * ximage)
   g_return_if_fail (ximage != NULL);
 
   ximagesink = ximage->ximagesink;
-  if (!ximagesink) {
+  if (G_UNLIKELY (ximagesink == NULL)) {
     GST_WARNING_OBJECT (ximagesink, "no sink found");
     goto beach;
   }
@@ -291,7 +293,6 @@ gst_ximage_buffer_get_type (void)
 
 /* X11 stuff */
 
-#ifdef HAVE_XSHM                /* Check that XShm calls actually work */
 static gboolean error_caught = FALSE;
 
 static int
@@ -300,11 +301,13 @@ gst_ximagesink_handle_xerror (Display * display, XErrorEvent * xevent)
   char error_msg[1024];
 
   XGetErrorText (display, xevent->error_code, error_msg, 1024);
-  GST_DEBUG ("ximagesink failed to use XShm calls. error: %s", error_msg);
+  GST_DEBUG ("ximagesink triggered an XError. error: %s", error_msg);
   error_caught = TRUE;
   return 0;
 }
 
+#ifdef HAVE_XSHM                /* Check that XShm calls actually work */
+
 static gboolean
 gst_ximagesink_check_xshm_calls (GstXImageSink * ximagesink,
     GstXContext * xcontext)
@@ -405,6 +408,7 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
   GstXImageBuffer *ximage = NULL;
   GstStructure *structure = NULL;
   gboolean succeeded = FALSE;
+  int (*handler) (Display *, XErrorEvent *);
 
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
 
@@ -422,14 +426,22 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
 
   g_mutex_lock (ximagesink->x_lock);
 
+  /* Setting an error handler to catch failure */
+  error_caught = FALSE;
+  handler = XSetErrorHandler (gst_ximagesink_handle_xerror);
+
 #ifdef HAVE_XSHM
   if (ximagesink->xcontext->use_xshm) {
     ximage->ximage = XShmCreateImage (ximagesink->xcontext->disp,
         ximagesink->xcontext->visual,
         ximagesink->xcontext->depth,
         ZPixmap, NULL, &ximage->SHMInfo, ximage->width, ximage->height);
-    if (!ximage->ximage) {
+    if (!ximage->ximage || error_caught) {
       g_mutex_unlock (ximagesink->x_lock);
+      /* Reset error handler */
+      error_caught = FALSE;
+      XSetErrorHandler (handler);
+      /* Push an error */
       GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE,
           ("Failed to create output image buffer of %dx%d pixels",
               ximage->width, ximage->height),
@@ -488,13 +500,19 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
   } else
 #endif /* HAVE_XSHM */
   {
+    guint allocsize;
+
     ximage->ximage = XCreateImage (ximagesink->xcontext->disp,
         ximagesink->xcontext->visual,
         ximagesink->xcontext->depth,
         ZPixmap, 0, NULL,
         ximage->width, ximage->height, ximagesink->xcontext->bpp, 0);
-    if (!ximage->ximage) {
+    if (!ximage->ximage || error_caught) {
       g_mutex_unlock (ximagesink->x_lock);
+      /* Reset error handler */
+      error_caught = FALSE;
+      XSetErrorHandler (handler);
+      /* Push an error */
       GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE,
           ("Failed to create output image buffer of %dx%d pixels",
               ximage->width, ximage->height),
@@ -503,12 +521,35 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
       goto beach;
     }
 
+    /* upstream will assume that rowstrides are multiples of 4, but this
+     * doesn't always seem to be the case with XCreateImage() */
+    if ((ximage->ximage->bytes_per_line % 4) != 0) {
+      GST_WARNING_OBJECT (ximagesink, "returned stride not a multiple of 4 as "
+          "usually assumed");
+    }
+
     /* we have to use the returned bytes_per_line for our image size */
     ximage->size = ximage->ximage->bytes_per_line * ximage->ximage->height;
-    ximage->ximage->data = g_malloc (ximage->size);
+
+    /* alloc a bit more for unexpected strides to avoid crashes upstream.
+     * FIXME: if we get an unrounded stride, the image will be displayed
+     * distorted, since all upstream elements assume a rounded stride */
+    allocsize =
+        GST_ROUND_UP_4 (ximage->ximage->bytes_per_line) *
+        ximage->ximage->height;
+    ximage->ximage->data = g_malloc (allocsize);
+    GST_LOG_OBJECT (ximagesink,
+        "non-XShm image size is %" G_GSIZE_FORMAT " (alloced: %u), width %d, "
+        "stride %d", ximage->size, allocsize, ximage->width,
+        ximage->ximage->bytes_per_line);
 
     XSync (ximagesink->xcontext->disp, FALSE);
   }
+
+  /* Reset error handler */
+  error_caught = FALSE;
+  XSetErrorHandler (handler);
+
   succeeded = TRUE;
 
   GST_BUFFER_DATA (ximage) = (guchar *) ximage->ximage->data;
@@ -535,14 +576,14 @@ gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink,
   g_return_if_fail (ximage != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
 
+  /* Hold the object lock to ensure the XContext doesn't disappear */
+  GST_OBJECT_LOCK (ximagesink);
+
   /* If the destroyed image is the current one we destroy our reference too */
   if (ximagesink->cur_image == ximage) {
     ximagesink->cur_image = NULL;
   }
 
-  /* Hold the object lock to ensure the XContext doesn't disappear */
-  GST_OBJECT_LOCK (ximagesink);
-
   /* We might have some buffers destroyed after changing state to NULL */
   if (!ximagesink->xcontext) {
     GST_DEBUG_OBJECT (ximagesink, "Destroying XImage after XContext");
@@ -627,18 +668,23 @@ gst_ximagesink_xwindow_draw_borders (GstXImageSink * ximagesink,
 }
 
 /* This function puts a GstXImageBuffer on a GstXImageSink's window */
-static void
+static gboolean
 gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
 {
   GstVideoRectangle src, dst, result;
   gboolean draw_border = FALSE;
 
-  g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
+  g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), FALSE);
 
   /* We take the flow_lock. If expose is in there we don't want to run
      concurrently from the data flow thread */
   g_mutex_lock (ximagesink->flow_lock);
 
+  if (G_UNLIKELY (ximagesink->xwindow == NULL)) {
+    g_mutex_unlock (ximagesink->flow_lock);
+    return FALSE;
+  }
+
   /* Draw borders when displaying the first frame. After this
      draw borders only on expose event. */
   if (!ximagesink->cur_image) {
@@ -663,7 +709,7 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
       ximage = ximagesink->cur_image;
     } else {
       g_mutex_unlock (ximagesink->flow_lock);
-      return;
+      return TRUE;
     }
   }
 
@@ -708,6 +754,8 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
   g_mutex_unlock (ximagesink->x_lock);
 
   g_mutex_unlock (ximagesink->flow_lock);
+
+  return TRUE;
 }
 
 static gboolean
@@ -771,9 +819,20 @@ gst_ximagesink_xwindow_new (GstXImageSink * ximagesink, gint width, gint height)
      ConfigureNotify. This takes away flickering of video when resizing. */
   XSetWindowBackgroundPixmap (ximagesink->xcontext->disp, xwindow->win, None);
 
-  XSelectInput (ximagesink->xcontext->disp, xwindow->win, ExposureMask |
-      StructureNotifyMask | PointerMotionMask | KeyPressMask |
-      KeyReleaseMask | ButtonPressMask | ButtonReleaseMask);
+  if (ximagesink->handle_events) {
+    Atom wm_delete;
+
+    XSelectInput (ximagesink->xcontext->disp, xwindow->win, ExposureMask |
+        StructureNotifyMask | PointerMotionMask | KeyPressMask |
+        KeyReleaseMask | ButtonPressMask | ButtonReleaseMask);
+
+    /* Tell the window manager we'd like delete client messages instead of
+     * being killed */
+    wm_delete = XInternAtom (ximagesink->xcontext->disp,
+        "WM_DELETE_WINDOW", False);
+    (void) XSetWMProtocols (ximagesink->xcontext->disp, xwindow->win,
+        &wm_delete, 1);
+  }
 
   xwindow->gc = XCreateGC (ximagesink->xcontext->disp, xwindow->win,
       0, &values);
@@ -863,44 +922,48 @@ static void
 gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
 {
   XEvent e;
+  guint pointer_x = 0, pointer_y = 0;
+  gboolean pointer_moved = FALSE;
+  gboolean exposed = FALSE, configured = FALSE;
 
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
 
-  {
-    guint pointer_x = 0, pointer_y = 0;
-    gboolean pointer_moved = FALSE;
-
-    /* Then we get all pointer motion events, only the last position is
-       interesting. */
-    g_mutex_lock (ximagesink->x_lock);
-    while (XCheckWindowEvent (ximagesink->xcontext->disp,
-            ximagesink->xwindow->win, PointerMotionMask, &e)) {
-      g_mutex_unlock (ximagesink->x_lock);
-
-      switch (e.type) {
-        case MotionNotify:
-          pointer_x = e.xmotion.x;
-          pointer_y = e.xmotion.y;
-          pointer_moved = TRUE;
-          break;
-        default:
-          break;
-      }
-
-      g_mutex_lock (ximagesink->x_lock);
-    }
+  /* Then we get all pointer motion events, only the last position is
+     interesting. */
+  g_mutex_lock (ximagesink->flow_lock);
+  g_mutex_lock (ximagesink->x_lock);
+  while (XCheckWindowEvent (ximagesink->xcontext->disp,
+          ximagesink->xwindow->win, PointerMotionMask, &e)) {
     g_mutex_unlock (ximagesink->x_lock);
+    g_mutex_unlock (ximagesink->flow_lock);
 
-    if (pointer_moved) {
-      GST_DEBUG ("ximagesink pointer moved over window at %d,%d",
-          pointer_x, pointer_y);
-      gst_navigation_send_mouse_event (GST_NAVIGATION (ximagesink),
-          "mouse-move", 0, pointer_x, pointer_y);
+    switch (e.type) {
+      case MotionNotify:
+        pointer_x = e.xmotion.x;
+        pointer_y = e.xmotion.y;
+        pointer_moved = TRUE;
+        break;
+      default:
+        break;
     }
+    g_mutex_lock (ximagesink->flow_lock);
+    g_mutex_lock (ximagesink->x_lock);
+  }
+
+  if (pointer_moved) {
+    g_mutex_unlock (ximagesink->x_lock);
+    g_mutex_unlock (ximagesink->flow_lock);
+
+    GST_DEBUG ("ximagesink pointer moved over window at %d,%d",
+        pointer_x, pointer_y);
+    gst_navigation_send_mouse_event (GST_NAVIGATION (ximagesink),
+        "mouse-move", 0, pointer_x, pointer_y);
+
+    g_mutex_lock (ximagesink->flow_lock);
+    g_mutex_lock (ximagesink->x_lock);
   }
 
   /* We get all remaining events on our window to throw them upstream */
-  g_mutex_lock (ximagesink->x_lock);
   while (XCheckWindowEvent (ximagesink->xcontext->disp,
           ximagesink->xwindow->win,
           KeyPressMask | KeyReleaseMask |
@@ -909,6 +972,7 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
 
     /* We lock only for the X function call */
     g_mutex_unlock (ximagesink->x_lock);
+    g_mutex_unlock (ximagesink->flow_lock);
 
     switch (e.type) {
       case ButtonPress:
@@ -931,12 +995,19 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
            events for interactivity/navigation */
         GST_DEBUG ("ximagesink key %d pressed over window at %d,%d",
             e.xkey.keycode, e.xkey.x, e.xkey.x);
+        g_mutex_lock (ximagesink->x_lock);
         keysym = XKeycodeToKeysym (ximagesink->xcontext->disp,
             e.xkey.keycode, 0);
+        g_mutex_unlock (ximagesink->x_lock);
         if (keysym != NoSymbol) {
+          char *key_str = NULL;
+
+          g_mutex_lock (ximagesink->x_lock);
+          key_str = XKeysymToString (keysym);
+          g_mutex_unlock (ximagesink->x_lock);
           gst_navigation_send_key_event (GST_NAVIGATION (ximagesink),
-              e.type == KeyPress ?
-              "key-press" : "key-release", XKeysymToString (keysym));
+              e.type == KeyPress ? "key-press" : "key-release", key_str);
+
         } else {
           gst_navigation_send_key_event (GST_NAVIGATION (ximagesink),
               e.type == KeyPress ? "key-press" : "key-release", "unknown");
@@ -946,34 +1017,63 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink)
         GST_DEBUG_OBJECT (ximagesink, "ximagesink unhandled X event (%d)",
             e.type);
     }
+    g_mutex_lock (ximagesink->flow_lock);
     g_mutex_lock (ximagesink->x_lock);
   }
-  g_mutex_unlock (ximagesink->x_lock);
 
-  {
-    gboolean exposed = FALSE;
-
-    g_mutex_lock (ximagesink->x_lock);
-    while (XCheckWindowEvent (ximagesink->xcontext->disp,
-            ximagesink->xwindow->win, ExposureMask, &e)) {
-      g_mutex_unlock (ximagesink->x_lock);
-
-      switch (e.type) {
-        case Expose:
-          exposed = TRUE;
-          break;
-        default:
-          break;
-      }
-
-      g_mutex_lock (ximagesink->x_lock);
+  while (XCheckWindowEvent (ximagesink->xcontext->disp,
+          ximagesink->xwindow->win, ExposureMask | StructureNotifyMask, &e)) {
+    switch (e.type) {
+      case Expose:
+        exposed = TRUE;
+        break;
+      case ConfigureNotify:
+        configured = TRUE;
+        break;
+      default:
+        break;
     }
+  }
+
+  if (ximagesink->handle_expose && (exposed || configured)) {
     g_mutex_unlock (ximagesink->x_lock);
+    g_mutex_unlock (ximagesink->flow_lock);
 
-    if (exposed) {
-      gst_ximagesink_expose (GST_X_OVERLAY (ximagesink));
+    gst_ximagesink_expose (GST_X_OVERLAY (ximagesink));
+
+    g_mutex_lock (ximagesink->flow_lock);
+    g_mutex_lock (ximagesink->x_lock);
+  }
+
+  /* Handle Display events */
+  while (XPending (ximagesink->xcontext->disp)) {
+    XNextEvent (ximagesink->xcontext->disp, &e);
+
+    switch (e.type) {
+      case ClientMessage:{
+        Atom wm_delete;
+
+        wm_delete = XInternAtom (ximagesink->xcontext->disp,
+            "WM_DELETE_WINDOW", False);
+        if (wm_delete == (Atom) e.xclient.data.l[0]) {
+          /* Handle window deletion by posting an error on the bus */
+          GST_ELEMENT_ERROR (ximagesink, RESOURCE, NOT_FOUND,
+              ("Output window was closed"), (NULL));
+
+          g_mutex_unlock (ximagesink->x_lock);
+          gst_ximagesink_xwindow_destroy (ximagesink, ximagesink->xwindow);
+          ximagesink->xwindow = NULL;
+          g_mutex_lock (ximagesink->x_lock);
+        }
+        break;
+      }
+      default:
+        break;
     }
   }
+
+  g_mutex_unlock (ximagesink->x_lock);
+  g_mutex_unlock (ximagesink->flow_lock);
 }
 
 static gpointer
@@ -996,7 +1096,7 @@ gst_ximagesink_event_thread (GstXImageSink * ximagesink)
 static void
 gst_ximagesink_calculate_pixel_aspect_ratio (GstXContext * xcontext)
 {
-  gint par[][2] = {
+  static const gint par[][2] = {
     {1, 1},                     /* regular screen */
     {16, 15},                   /* PAL TV */
     {11, 10},                   /* 525 line Rec.601 video */
@@ -1217,7 +1317,7 @@ gst_ximagesink_xcontext_clear (GstXImageSink * ximagesink)
 
   g_mutex_unlock (ximagesink->x_lock);
 
-  g_free (ximagesink->xcontext);
+  g_free (xcontext);
 }
 
 static void
@@ -1296,6 +1396,7 @@ gst_ximagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
   GST_DEBUG_OBJECT (ximagesink, "intersection returned %" GST_PTR_FORMAT,
       intersection);
   if (gst_caps_is_empty (intersection)) {
+    gst_caps_unref (intersection);
     return FALSE;
   }
 
@@ -1331,17 +1432,28 @@ gst_ximagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
   ximagesink->fps_d = gst_value_get_fraction_denominator (fps);
 
   /* Notify application to set xwindow id now */
+  g_mutex_lock (ximagesink->flow_lock);
   if (!ximagesink->xwindow) {
+    g_mutex_unlock (ximagesink->flow_lock);
     gst_x_overlay_prepare_xwindow_id (GST_X_OVERLAY (ximagesink));
+  } else {
+    g_mutex_unlock (ximagesink->flow_lock);
   }
 
   /* Creating our window and our image */
-  g_assert (GST_VIDEO_SINK_WIDTH (ximagesink) > 0);
-  g_assert (GST_VIDEO_SINK_HEIGHT (ximagesink) > 0);
+  if (GST_VIDEO_SINK_WIDTH (ximagesink) <= 0 ||
+      GST_VIDEO_SINK_HEIGHT (ximagesink) <= 0) {
+    GST_ELEMENT_ERROR (ximagesink, CORE, NEGOTIATION, (NULL),
+        ("Invalid image size."));
+    return FALSE;
+  }
+
+  g_mutex_lock (ximagesink->flow_lock);
   if (!ximagesink->xwindow) {
     ximagesink->xwindow = gst_ximagesink_xwindow_new (ximagesink,
         GST_VIDEO_SINK_WIDTH (ximagesink), GST_VIDEO_SINK_HEIGHT (ximagesink));
   }
+  g_mutex_unlock (ximagesink->flow_lock);
 
   /* If our ximage has changed we destroy it, next chain iteration will create
      a new one */
@@ -1398,6 +1510,10 @@ gst_ximagesink_change_state (GstElement * element, GstStateChange transition)
       g_mutex_unlock (ximagesink->x_lock);
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
+      g_mutex_lock (ximagesink->flow_lock);
+      if (ximagesink->xwindow)
+        gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
+      g_mutex_unlock (ximagesink->flow_lock);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -1406,42 +1522,18 @@ gst_ximagesink_change_state (GstElement * element, GstStateChange transition)
   }
 
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-  if (ret == GST_STATE_CHANGE_FAILURE)
-    return ret;
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      if (ximagesink->xwindow)
-        gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
       ximagesink->fps_n = 0;
       ximagesink->fps_d = 1;
       GST_VIDEO_SINK_WIDTH (ximagesink) = 0;
       GST_VIDEO_SINK_HEIGHT (ximagesink) = 0;
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
-      GST_OBJECT_LOCK (ximagesink);
-      ximagesink->running = FALSE;
-      GST_OBJECT_UNLOCK (ximagesink);
-
-      if (ximagesink->ximage) {
-        gst_buffer_unref (ximagesink->ximage);
-        ximagesink->ximage = NULL;
-      }
-      if (ximagesink->cur_image) {
-        gst_buffer_unref (ximagesink->cur_image);
-        ximagesink->cur_image = NULL;
-      }
-
-      gst_ximagesink_bufferpool_clear (ximagesink);
-
-      if (ximagesink->xwindow) {
-        gst_ximagesink_xwindow_destroy (ximagesink, ximagesink->xwindow);
-        ximagesink->xwindow = NULL;
-      }
-
-      gst_ximagesink_xcontext_clear (ximagesink);
+      gst_ximagesink_reset (ximagesink);
       break;
     default:
       break;
@@ -1486,7 +1578,8 @@ gst_ximagesink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
      put the ximage which is in the PRIVATE pointer */
   if (GST_IS_XIMAGE_BUFFER (buf)) {
     GST_LOG_OBJECT (ximagesink, "buffer from our pool, writing directly");
-    gst_ximagesink_ximage_put (ximagesink, GST_XIMAGE_BUFFER (buf));
+    if (!gst_ximagesink_ximage_put (ximagesink, GST_XIMAGE_BUFFER (buf)))
+      goto no_window;
   } else {
     /* Else we have to copy the data into our private image, */
     /* if we have one... */
@@ -1512,7 +1605,8 @@ gst_ximagesink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
     }
     memcpy (GST_BUFFER_DATA (ximagesink->ximage), GST_BUFFER_DATA (buf),
         MIN (GST_BUFFER_SIZE (buf), ximagesink->ximage->size));
-    gst_ximagesink_ximage_put (ximagesink, ximagesink->ximage);
+    if (!gst_ximagesink_ximage_put (ximagesink, ximagesink->ximage))
+      goto no_window;
   }
 
   return GST_FLOW_OK;
@@ -1524,6 +1618,12 @@ no_ximage:
     GST_DEBUG ("could not create image");
     return GST_FLOW_ERROR;
   }
+no_window:
+  {
+    /* No Window available to put our image into */
+    GST_WARNING_OBJECT (ximagesink, "could not output image - no window");
+    return GST_FLOW_ERROR;
+  }
 }
 
 /* Buffer management
@@ -1770,25 +1870,25 @@ gst_ximagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
   GstXWindow *xwindow = NULL;
   XWindowAttributes attr;
 
-  /* If we already use that window return */
-  if (ximagesink->xwindow && (xwindow_id == ximagesink->xwindow->win))
-    return;
-
-  /* If the element has not initialized the X11 context try to do so */
-  if (!ximagesink->xcontext)
-    ximagesink->xcontext = gst_ximagesink_xcontext_get (ximagesink);
-
-  if (!ximagesink->xcontext) {
-    GST_WARNING_OBJECT (ximagesink,
-        "ximagesink was unable to obtain the X11 context.");
-    return;
-  }
-
   /* We acquire the stream lock while setting this window in the element.
      We are basically cleaning tons of stuff replacing the old window, putting
      images while we do that would surely crash */
   g_mutex_lock (ximagesink->flow_lock);
 
+  /* If we already use that window return */
+  if (ximagesink->xwindow && (xwindow_id == ximagesink->xwindow->win)) {
+    g_mutex_unlock (ximagesink->flow_lock);
+    return;
+  }
+
+  /* If the element has not initialized the X11 context try to do so */
+  if (!ximagesink->xcontext &&
+      !(ximagesink->xcontext = gst_ximagesink_xcontext_get (ximagesink))) {
+    g_mutex_unlock (ximagesink->flow_lock);
+    /* we have thrown a GST_ELEMENT_ERROR now */
+    return;
+  }
+
   /* If a window is there already we destroy it */
   if (ximagesink->xwindow) {
     gst_ximagesink_xwindow_destroy (ximagesink, ximagesink->xwindow);
@@ -1816,9 +1916,11 @@ gst_ximagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
     xwindow->width = attr.width;
     xwindow->height = attr.height;
     xwindow->internal = FALSE;
-    XSelectInput (ximagesink->xcontext->disp, xwindow->win, ExposureMask |
-        StructureNotifyMask | PointerMotionMask | KeyPressMask |
-        KeyReleaseMask);
+    if (ximagesink->handle_events) {
+      XSelectInput (ximagesink->xcontext->disp, xwindow->win, ExposureMask |
+          StructureNotifyMask | PointerMotionMask | KeyPressMask |
+          KeyReleaseMask);
+    }
 
     xwindow->gc = XCreateGC (ximagesink->xcontext->disp, xwindow->win, 0, NULL);
     g_mutex_unlock (ximagesink->x_lock);
@@ -1835,17 +1937,51 @@ gst_ximagesink_expose (GstXOverlay * overlay)
 {
   GstXImageSink *ximagesink = GST_XIMAGESINK (overlay);
 
-  if (!ximagesink->xwindow)
-    return;
-
   gst_ximagesink_ximage_put (ximagesink, NULL);
 }
 
+static void
+gst_ximagesink_set_event_handling (GstXOverlay * overlay,
+    gboolean handle_events)
+{
+  GstXImageSink *ximagesink = GST_XIMAGESINK (overlay);
+
+  ximagesink->handle_events = handle_events;
+
+  g_mutex_lock (ximagesink->flow_lock);
+
+  if (G_UNLIKELY (!ximagesink->xwindow)) {
+    g_mutex_unlock (ximagesink->flow_lock);
+    return;
+  }
+
+  g_mutex_lock (ximagesink->x_lock);
+
+  if (handle_events) {
+    if (ximagesink->xwindow->internal) {
+      XSelectInput (ximagesink->xcontext->disp, ximagesink->xwindow->win,
+          ExposureMask | StructureNotifyMask | PointerMotionMask |
+          KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask);
+    } else {
+      XSelectInput (ximagesink->xcontext->disp, ximagesink->xwindow->win,
+          ExposureMask | StructureNotifyMask | PointerMotionMask |
+          KeyPressMask | KeyReleaseMask);
+    }
+  } else {
+    XSelectInput (ximagesink->xcontext->disp, ximagesink->xwindow->win, 0);
+  }
+
+  g_mutex_unlock (ximagesink->x_lock);
+
+  g_mutex_unlock (ximagesink->flow_lock);
+}
+
 static void
 gst_ximagesink_xoverlay_init (GstXOverlayClass * iface)
 {
   iface->set_xwindow_id = gst_ximagesink_set_xwindow_id;
   iface->expose = gst_ximagesink_expose;
+  iface->handle_events = gst_ximagesink_set_event_handling;
 }
 
 /* =========================================== */
@@ -1901,6 +2037,13 @@ gst_ximagesink_set_property (GObject * object, guint prop_id,
       }
     }
       break;
+    case PROP_HANDLE_EVENTS:
+      gst_ximagesink_set_event_handling (GST_X_OVERLAY (ximagesink),
+          g_value_get_boolean (value));
+      break;
+    case PROP_HANDLE_EXPOSE:
+      ximagesink->handle_expose = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1931,12 +2074,47 @@ gst_ximagesink_get_property (GObject * object, guint prop_id,
       if (ximagesink->par)
         g_value_transform (ximagesink->par, value);
       break;
+    case PROP_HANDLE_EVENTS:
+      g_value_set_boolean (value, ximagesink->handle_events);
+      break;
+    case PROP_HANDLE_EXPOSE:
+      g_value_set_boolean (value, ximagesink->handle_expose);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
+static void
+gst_ximagesink_reset (GstXImageSink * ximagesink)
+{
+  GST_OBJECT_LOCK (ximagesink);
+  ximagesink->running = FALSE;
+  GST_OBJECT_UNLOCK (ximagesink);
+
+  if (ximagesink->ximage) {
+    gst_buffer_unref (ximagesink->ximage);
+    ximagesink->ximage = NULL;
+  }
+  if (ximagesink->cur_image) {
+    gst_buffer_unref (ximagesink->cur_image);
+    ximagesink->cur_image = NULL;
+  }
+
+  gst_ximagesink_bufferpool_clear (ximagesink);
+
+  g_mutex_lock (ximagesink->flow_lock);
+  if (ximagesink->xwindow) {
+    gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
+    gst_ximagesink_xwindow_destroy (ximagesink, ximagesink->xwindow);
+    ximagesink->xwindow = NULL;
+  }
+  g_mutex_unlock (ximagesink->flow_lock);
+
+  gst_ximagesink_xcontext_clear (ximagesink);
+}
+
 static void
 gst_ximagesink_finalize (GObject * object)
 {
@@ -1944,6 +2122,8 @@ gst_ximagesink_finalize (GObject * object)
 
   ximagesink = GST_XIMAGESINK (object);
 
+  gst_ximagesink_reset (ximagesink);
+
   if (ximagesink->display_name) {
     g_free (ximagesink->display_name);
     ximagesink->display_name = NULL;
@@ -1993,6 +2173,8 @@ gst_ximagesink_init (GstXImageSink * ximagesink)
 
   ximagesink->synchronous = FALSE;
   ximagesink->keep_aspect = FALSE;
+  ximagesink->handle_events = TRUE;
+  ximagesink->handle_expose = TRUE;
 }
 
 static void
@@ -2037,6 +2219,14 @@ gst_ximagesink_class_init (GstXImageSinkClass * klass)
   g_object_class_install_property (gobject_class, PROP_PIXEL_ASPECT_RATIO,
       g_param_spec_string ("pixel-aspect-ratio", "Pixel Aspect Ratio",
           "The pixel aspect ratio of the device", "1/1", G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_HANDLE_EVENTS,
+      g_param_spec_boolean ("handle-events", "Handle XEvents",
+          "When enabled, XEvents will be selected and handled", TRUE,
+          G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_HANDLE_EXPOSE,
+      g_param_spec_boolean ("handle-expose", "Handle expose", "When enabled, "
+          "the current frame will always be drawn in response to X Expose "
+          "events", TRUE, G_PARAM_READWRITE));
 
   gstelement_class->change_state = gst_ximagesink_change_state;
 
